@@ -2,7 +2,7 @@
 // Name:        src/gtk/radiobox.cpp
 // Purpose:
 // Author:      Robert Roebling
-// Id:          $Id: radiobox.cpp 45050 2007-03-25 02:03:27Z VZ $
+// Id:          $Id: radiobox.cpp 58188 2009-01-17 20:38:43Z JS $
 // Copyright:   (c) 1998 Robert Roebling
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -49,7 +49,7 @@ public:
 //-----------------------------------------------------------------------------
 
 #include "wx/listimpl.cpp"
-WX_DEFINE_LIST( wxRadioBoxButtonsInfoList );
+WX_DEFINE_LIST(wxRadioBoxButtonsInfoList)
 
 extern bool          g_blockEventsOnDrag;
 
@@ -473,6 +473,8 @@ void wxRadioBox::SetString(unsigned int item, const wxString& label)
 
 bool wxRadioBox::Enable( bool enable )
 {
+    bool isEnabled = IsEnabled();
+
     if ( !wxControl::Enable( enable ) )
         return false;
 
@@ -487,6 +489,9 @@ bool wxRadioBox::Enable( bool enable )
         node = node->GetNext();
     }
 
+    if (!isEnabled && enable)
+        wxGtkFixSensitivity(this);
+
     return true;
 }
 
