@@ -5,7 +5,7 @@
 // Author:      Robin Dunn
 //
 // Created:     1-July-1997
-// RCS-ID:      $Id: helpers.cpp 48467 2007-08-30 19:55:14Z RD $
+// RCS-ID:      $Id: helpers.cpp 53375 2008-04-27 03:59:28Z RD $
 // Copyright:   (c) 1998 by Total Control Software
 // Licence:     wxWindows license
 /////////////////////////////////////////////////////////////////////////////
@@ -405,7 +405,7 @@ void wxPyApp::_BootstrapApp()
 {
     static      bool haveInitialized = false;
     bool        result;
-    wxPyBlock_t     blocked;
+    wxPyBlock_t blocked;
     PyObject*   retval = NULL;
     PyObject*   pyint  = NULL;
 
@@ -484,7 +484,6 @@ void wxPyApp::_BootstrapApp()
             goto error;
     }
     if (wxPyCBH_findCallback(m_myInst, "OnInit")) {
-
         PyObject* method = m_myInst.GetLastFound();
         PyObject* argTuple = PyTuple_New(0);
         retval = PyEval_CallObject(method, argTuple);
@@ -1815,7 +1814,10 @@ void wxPyCallbackHelper::clearRecursionGuard(PyObject* method) const
 {
     PyFunctionObject* func = (PyFunctionObject*)PyMethod_Function(method);
     if (PyObject_HasAttr(m_self, func->func_name)) {
-        PyObject_DelAttr(m_self, func->func_name);
+        PyObject* attr = PyObject_GetAttr(m_self, func->func_name);
+        if ( attr == Py_None )
+	    PyObject_DelAttr(m_self, func->func_name);
+        Py_DECREF(attr);
     }
 }
 
@@ -2006,29 +2008,6 @@ wxPyCommandEvent::~wxPyCommandEvent() {
 
 //---------------------------------------------------------------------------
 //---------------------------------------------------------------------------
-// Convert a wxList to a Python List, only works for lists of wxObjects
-
-PyObject* wxPy_ConvertList(wxListBase* listbase) {
-    wxList*     list = (wxList*)listbase;  // this is probably bad...
-    PyObject*   pyList;
-    PyObject*   pyObj;
-    wxObject*   wxObj;
-    wxNode*     node = list->GetFirst();
-
-    wxPyBlock_t blocked = wxPyBeginBlockThreads();
-    pyList = PyList_New(0);
-    while (node) {
-        wxObj = node->GetData();
-        pyObj = wxPyMake_wxObject(wxObj,false);
-        PyList_Append(pyList, pyObj);
-        Py_DECREF(pyObj);  // the Append also does an INCREF, that's one more than we need.
-        node = node->GetNext();
-    }
-    wxPyEndBlockThreads(blocked);
-    return pyList;
-}
-
-//----------------------------------------------------------------------
 
 long wxPyGetWinHandle(wxWindow* win) {
 
@@ -2068,8 +2047,9 @@ wxString* wxString_in_helper(PyObject* source) {
     target = new wxString();
     size_t len = PyUnicode_GET_SIZE(uni);
     if (len) {
-        PyUnicode_AsWideChar((PyUnicodeObject*)uni, target->GetWriteBuf(len), len);
-        target->UngetWriteBuf(len);
+        PyUnicode_AsWideChar((PyUnicodeObject*)uni,
+                             wxStringBuffer(*target, len),
+                             len);
     }
 
     if (PyString_Check(source))
@@ -2121,8 +2101,9 @@ wxString Py2wxString(PyObject* source)
     }
     size_t len = PyUnicode_GET_SIZE(uni);
     if (len) {
-        PyUnicode_AsWideChar((PyUnicodeObject*)uni, target.GetWriteBuf(len), len);
-        target.UngetWriteBuf();
+        PyUnicode_AsWideChar((PyUnicodeObject*)uni,
+                             wxStringBuffer(target, len),
+                             len);
     }
 
     if (!PyUnicode_Check(source))
@@ -3053,7 +3034,8 @@ PyObject* wxPyImageHandler::py_Image(wxImage* image) {
 }
 
 PyObject* wxPyImageHandler::py_OutputStream(wxOutputStream* stream) {
-    return wxPyConstructObject(stream, wxT("wxOutputStream"), 0);
+    return wxPyConstructObject(new wxPyOutputStream(stream),
+                               wxT("wxPyOutputStream"), 0);
 }
 
 wxPyImageHandler::wxPyImageHandler():
