@@ -24,7 +24,7 @@ access the instance via the Publisher object available from the module::
 
 :Author:      Oliver Schoenborn
 :Since:       Apr 2004
-:Version:     $Id: pubsub.py,v 1.5.2.3 2006/06/10 23:46:47 RD Exp $
+:Version:     $Id: pubsub.py 53834 2008-05-30 02:27:12Z RD $
 :Copyright:   \(c) 2004 Oliver Schoenborn
 :License:     wxWidgets
 """
@@ -86,16 +86,19 @@ def _paramMinCount(callableObject):
     is number of default arguments. The 'self' parameter, in the case
     of methods, is not counted.
     """
-    if type(callableObject) is InstanceType:
-        min, d = _paramMinCountFunc(callableObject.__call__.im_func)
-        return min-1, d
-    elif ismethod(callableObject):
-        min, d = _paramMinCountFunc(callableObject.im_func)
-        return min-1, d
-    elif isfunction(callableObject):
-        return _paramMinCountFunc(callableObject)
-    else:
-        raise 'Cannot determine type of callable: '+repr(callableObject)
+    try:
+        func = callableObject.__call__.im_func
+    except AttributeError:
+        try:
+            func = callableObject.im_func
+        except AttributeError:
+            try:
+                return _paramMinCountFunc(callableObject)
+            except exc:
+                raise 'Cannot determine type of callable: %s' % repr(callableObject)
+ 
+    min, d = _paramMinCountFunc(func)
+    return min-1, d
 
 
 def _tupleize(items):
