@@ -9,8 +9,8 @@
 
 """Launch User Interface"""
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: launch.py 54046 2008-06-09 02:28:35Z CJP $"
-__revision__ = "$Revision: 54046 $"
+__svnid__ = "$Id: launch.py 57241 2008-12-11 03:33:30Z CJP $"
+__revision__ = "$Revision: 57241 $"
 __version__ = "0.1"
 
 #-----------------------------------------------------------------------------#
@@ -45,9 +45,11 @@ LAUNCH_KEY = 'Launch.Config'
 
 # Custom Messages
 MSG_RUN_LAUNCH = ('launch', 'run')
+MSG_RUN_LAST = ('launch', 'runlast')
 
-# Value reqest messages
+# Value request messages
 REQUEST_ACTIVE = 'Launch.IsActive'
+REQUEST_RELAUNCH = 'Launch.CanRelaunch'
 
 _ = wx.GetTranslation
 #-----------------------------------------------------------------------------#
@@ -60,14 +62,16 @@ class LaunchWindow(ctrlbox.ControlBox):
         # Attributes
         self._mw = self.__FindMainWindow()
         self._buffer = OutputDisplay(self)
-        self._slbl = None # Created in __DoLayout
+        self._fnames = list()
+        self._chFiles = None # Created in __DoLayout
         self._worker = None
         self._busy = False
         self._isready = False
         self._config = dict(file='', lang=0,
                             cfile='', clang=0,
                             last='', lastlang=0,
-                            prelang=0)
+                            prelang=0, largs='',
+                            lcmd='')
         self._prefs = Profile_Get('Launch.Prefs', default=None)
 
         # Setup
@@ -91,23 +95,37 @@ class LaunchWindow(ctrlbox.ControlBox):
         self.UpdateBufferColors()
         cbuffer = self._mw.GetNotebook().GetCurrentCtrl()
         self.SetupControlBar(cbuffer)
+        self._config['lang'] = GetLangIdFromMW(self._mw)
+        self.UpdateCurrentFiles(self._config['lang'])
+        self.SetFile(GetTextBuffer(self._mw).GetFileName())
+
+        # Setup filetype settings
+        self.RefreshControlBar()
 
         # Event Handlers
         self.Bind(wx.EVT_BUTTON, self.OnButton)
+        self.Bind(wx.EVT_CHOICE, self.OnChoice)
         ed_msg.Subscribe(self.OnPageChanged, ed_msg.EDMSG_UI_NB_CHANGED)
         ed_msg.Subscribe(self.OnFileOpened, ed_msg.EDMSG_FILE_OPENED)
         ed_msg.Subscribe(self.OnThemeChanged, ed_msg.EDMSG_THEME_CHANGED)
+        ed_msg.Subscribe(self.OnLexerChange, ed_msg.EDMSG_UI_STC_LEXER)
         ed_msg.Subscribe(self.OnConfigExit, cfgdlg.EDMSG_LAUNCH_CFG_EXIT)
         ed_msg.Subscribe(self.OnRunMsg, MSG_RUN_LAUNCH)
+        ed_msg.Subscribe(self.OnRunLastMsg, MSG_RUN_LAST)
         ed_msg.RegisterCallback(self._CanLaunch, REQUEST_ACTIVE)
+        ed_msg.RegisterCallback(self._CanReLaunch, REQUEST_RELAUNCH)
 
     def __del__(self):
         ed_msg.Unsubscribe(self.OnPageChanged)
+        ed_msg.Unsubscribe(self.OnFileOpened)
         ed_msg.Unsubscribe(self.OnThemeChanged)
+        ed_msg.Unsubscribe(self.OnLexerChange)
         ed_msg.Unsubscribe(self.OnConfigExit)
         ed_msg.Unsubscribe(self.OnRunMsg)
-        ed_msg.UnRegisterCallback(self.CanLaunch)
-        super(LaunchWindow).__del__()
+        ed_msg.Unsubscribe(self.OnRunLastMsg)
+        ed_msg.UnRegisterCallback(self._CanLaunch)
+        ed_msg.UnRegisterCallback(self._CanReLaunch)
+        super(LaunchWindow, self).__del__()
 
     def __DoLayout(self):
         """Layout the window"""
@@ -132,8 +150,8 @@ class LaunchWindow(ctrlbox.ControlBox):
 
         # Script Label
         ctrlbar.AddControl((5, 5), wx.ALIGN_LEFT)
-        self._slbl = wx.StaticText(ctrlbar, label="")
-        ctrlbar.AddControl(self._slbl, wx.ALIGN_LEFT)
+        self._chFiles = wx.Choice(ctrlbar, choices=[''])
+        ctrlbar.AddControl(self._chFiles, wx.ALIGN_LEFT)
 
         # Args
         ctrlbar.AddControl((5, 5), wx.ALIGN_LEFT)
@@ -192,14 +210,20 @@ class LaunchWindow(ctrlbox.ControlBox):
             val = ed_msg.NullValue()
         return val
 
+    def _CanReLaunch(self):
+        """Method to use with RegisterCallback for getting status"""
+        val = self.CanLaunch()
+        if not val or not len(self._config['last']):
+            val = ed_msg.NullValue()
+        return val
+
     def CanLaunch(self):
         """Can the launch window run or not
         @return: bool
 
         """
         parent = self.GetParent()
-        return parent.GetCurrentPage() == self and \
-               parent.GetParent().IsActive() and self._isready
+        return parent.GetParent().IsActive() and self._isready
 
     def GetFile(self):
         """Get the file that is currently set to be run
@@ -243,6 +267,25 @@ class LaunchWindow(ctrlbox.ControlBox):
         else:
             evt.Skip()
 
+    def OnChoice(self, evt):
+        """Handle events from the Choice controls
+        @param evt: wx.CommandEvent
+
+        """
+        e_id = evt.GetId()
+        e_sel = evt.GetSelection()
+        if e_id == self._chFiles.GetId():
+            fname = self._fnames[e_sel]
+            self.SetFile(fname)
+            self._chFiles.SetToolTipString(fname)
+        elif e_id == ID_EXECUTABLE:
+            e_obj = evt.GetEventObject()
+            handler = handlers.GetHandlerById(self._config['lang'])
+            cmd = e_obj.GetStringSelection()
+            e_obj.SetToolTipString(handler.GetCommand(cmd))
+        else:
+            evt.Skip()
+
     def OnConfigExit(self, msg):
         """Update current state when the config dialog has been closed
         @param msg: Message Object
@@ -262,13 +305,36 @@ class LaunchWindow(ctrlbox.ControlBox):
         if not self._mw.IsActive():
             return
 
+        # Update the file choice control
+        self._config['lang'] = GetLangIdFromMW(self._mw)
+        self.UpdateCurrentFiles(self._config['lang'])
+
         fname = msg.GetData()
         self.SetFile(fname)
 
         # Setup filetype settings
-        self._config['lang'] = GetLangIdFromMW(self._mw)
         self.RefreshControlBar()
 
+    def OnLexerChange(self, msg):
+        """Update the status of the currently associated file
+        when a file is saved. Used for updating after a file type has
+        changed due to a save action.
+        @param msg: Message object
+
+        """
+        if not self._mw.IsActive():
+            return
+
+        mdata = msg.GetData()
+        # For backwards compatibility with older message format
+        if mdata is None:
+            return
+
+        fname, ftype = msg.GetData()
+        if ftype != self._config['lang']:
+            self.UpdateCurrentFiles(ftype)
+            self.SetControlBarState(fname, ftype)
+
     def OnPageChanged(self, msg):
         """Update the status of the currently associated file
         when the page changes in the main notebook.
@@ -281,6 +347,7 @@ class LaunchWindow(ctrlbox.ControlBox):
 
         mval = msg.GetData()
         ctrl = mval[0].GetCurrentCtrl()
+        self.UpdateCurrentFiles(ctrl.GetLangId())
         if hasattr(ctrl, 'GetFileName'):
             self.SetupControlBar(ctrl)
 
@@ -291,8 +358,34 @@ class LaunchWindow(ctrlbox.ControlBox):
 
         """
         if self.CanLaunch():
+            shelf = self._mw.GetShelf()
+            shelf.RaiseWindow(self)
             self.StartStopProcess()
 
+    def OnRunLastMsg(self, msg):
+        """Re-run the last run program.
+        @param msg: MSG_RUN_LAST
+
+        """
+        if self.CanLaunch():
+            fname, ftype = self.GetLastRun()
+            # If there is no last run file return
+            if not len(fname):
+                return
+
+            shelf = self._mw.GetShelf()
+            self.UpdateCurrentFiles(ftype)
+            self.SetFile(fname)
+            self.RefreshControlBar()
+            shelf.RaiseWindow(self)
+
+            if self._prefs.get('autoclear'):
+                self._buffer.Clear()
+
+            self.SetProcessRunning(True)
+
+            self.Run(fname, self._config['lcmd'], self._config['largs'], ftype)
+
     def OnThemeChanged(self, msg):
         """Update icons when the theme has been changed
         @param msg: Message Object
@@ -324,22 +417,52 @@ class LaunchWindow(ctrlbox.ControlBox):
         # Set control states
         csel = exe_ch.GetStringSelection()
         exe_ch.SetItems(cmds)
+        if len(cmds):
+            exe_ch.SetToolTipString(handler.GetCommand(cmds[0]))
+
         util.Log("[Launch][info] Found commands %s" % str(cmds))
         if handler.GetName() != handlers.DEFAULT_HANDLER and len(self.GetFile()):
-            exe_ch.Enable()
-            args_txt.Enable()
-            run_btn.Enable()
+            for ctrl in (exe_ch, args_txt, run_btn, self._chFiles):
+                ctrl.Enable()
             self._isready = True
+
             if self._config['lang'] == self._config['prelang'] and len(csel):
                 exe_ch.SetStringSelection(csel)
             else:
-                exe_ch.SetStringSelection(handler.GetDefault())
+                csel = handler.GetDefault()
+                exe_ch.SetStringSelection(csel)
+
+            exe_ch.SetToolTipString(handler.GetCommand(csel))
             self.GetControlBar().Layout()
         else:
             self._isready = False
-            run_btn.Disable()
-            args_txt.Disable()
-            exe_ch.Disable()
+            for ctrl in (exe_ch, args_txt, run_btn, self._chFiles):
+                ctrl.Disable()
+
+    def Run(self, fname, cmd, args, ftype):
+        """Run the given file
+        @param fname: File path
+        @param cmd: Command to run on file
+        @param args: Executable arguments
+        @param ftype: File type id
+
+        """
+        # Find and save the file if it is modified
+        nb = self._mw.GetNotebook()
+        for ctrl in nb.GetTextControls():
+            tname = ctrl.GetFileName()
+            if fname == tname:
+                if ctrl.GetModify():
+                    ctrl.SaveFile(tname)
+                    break
+
+        handler = handlers.GetHandlerById(ftype)
+        path, fname = os.path.split(fname)
+        self._worker = outbuff.ProcessThread(self._buffer,
+                                             cmd, fname,
+                                             args, path,
+                                             handler.GetEnvironment())
+        self._worker.start()
 
     def StartStopProcess(self):
         """Run or abort the context of the current process if possible"""
@@ -351,13 +474,11 @@ class LaunchWindow(ctrlbox.ControlBox):
             util.Log("[Launch][info] Starting process")
             handler = handlers.GetHandlerById(self._config['lang'])
             cmd = self.FindWindowById(ID_EXECUTABLE).GetStringSelection()
+            self._config['lcmd'] = cmd
             cmd = handler.GetCommand(cmd)
-            path, fname = os.path.split(self._config['file'])
             args = self.FindWindowById(ID_ARGS).GetValue().split()
-            self._worker = outbuff.ProcessThread(self._buffer, cmd, fname,
-                                                 args, path,
-                                                 handler.GetEnvironment())
-            self._worker.start()
+            self._config['largs'] = args
+            self.Run(self._config['file'], cmd, args, self._config['lang'])
         else:
             util.Log("[Launch][info] Aborting process")
             self._worker.Abort()
@@ -368,9 +489,9 @@ class LaunchWindow(ctrlbox.ControlBox):
         @param fname: file path
 
         """
+        # Set currently selected file
         self._config['file'] = fname
-        sname = os.path.split(fname)[1]
-        self._slbl.SetLabel(_("file") + ": " + sname)
+        self._chFiles.SetStringSelection(os.path.split(fname)[1])
         self.GetControlBar().Layout()
 
     def SetLangId(self, langid):
@@ -423,7 +544,9 @@ class LaunchWindow(ctrlbox.ControlBox):
         """
         fname = ctrl.GetFileName()
         lang_id = ctrl.GetLangId()
+        self.SetControlBarState(fname, lang_id)
 
+    def SetControlBarState(self, fname, lang_id):
         # Don't update the bars status if the buffer is busy
         if self._buffer.IsRunning():
             self._config['cfile'] = fname
@@ -451,6 +574,28 @@ class LaunchWindow(ctrlbox.ControlBox):
         self._buffer.SetInfoColor(colors['infof'], colors['infob'])
         self._buffer.SetWarningColor(colors['warnf'], colors['warnb'])
 
+    def UpdateCurrentFiles(self, lang_id):
+        """Update the current set of open files that are of the same
+        type.
+        @param lang_id: Editra filetype id
+        @postcondition: all open files that are of the same type are set
+                        and stored in the file choice control.
+
+        """
+        self._fnames = list()
+        for txt_ctrl in self._mw.GetNotebook().GetTextControls():
+            if lang_id == txt_ctrl.GetLangId():
+                self._fnames.append(txt_ctrl.GetFileName())
+
+        items = [ os.path.basename(fname) for fname in self._fnames ]
+        try:
+            self._chFiles.SetItems(items)
+            if len(self._fnames):
+                self._chFiles.SetToolTipString(self._fnames[0])
+        except TypeError:
+            util.Log("[Launch][err] UpdateCurrent Files: " + str(items))
+            self._chFiles.SetItems([''])
+
 #-----------------------------------------------------------------------------#
 
 class OutputDisplay(outbuff.OutputBuffer, outbuff.ProcessBufferMixin):
