@@ -2,7 +2,7 @@
 # Name: ed_menu.py                                                            #
 # Purpose: Editra's Menubar and Menu related classes                          #
 # Author: Cody Precord <cprecord@editra.org>                                  #
-# Copyright: (c) 2008 Cody Precord <staff@editra.org>                         #
+# Copyright: (c) 2007-2008 Cody Precord <staff@editra.org>                    #
 # License: wxWindows License                                                  #
 ###############################################################################
 
@@ -14,8 +14,8 @@ for managing keybindings and profiles is also provided by this module.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_menu.py 54247 2008-06-15 17:52:01Z CJP $"
-__revision__ = "$Revision: 54247 $"
+__svnid__ = "$Id: ed_menu.py 58836 2009-02-12 03:35:25Z CJP $"
+__revision__ = "$Revision: 58836 $"
 
 #--------------------------------------------------------------------------#
 # Dependancies
@@ -222,7 +222,7 @@ class KeyBinder(object):
             shortcut = u"+".join(rbind)
             if len(shortcut):
                 shortcut = u"\t" + shortcut
-        return shortcut
+        return unicode(shortcut)
 
     @classmethod
     def GetCurrentProfile(cls):
@@ -246,7 +246,7 @@ class KeyBinder(object):
         @return: list of strings
 
         """
-        recs = util.GetResourceFiles('cache', trim=True, get_all=False,
+        recs = util.GetResourceFiles(u'cache', trim=True, get_all=False,
                                      suffix='.ekeys', title=False)
         if recs == -1:
             recs = list()
@@ -282,10 +282,7 @@ class KeyBinder(object):
         @return: tuple
 
         """
-        if cls.cprofile is None:
-            return _DEFAULT_BINDING.get(item_id, None)
-        else:
-            return cls.keyprofile.get(item_id, None)
+        return cls.keyprofile.get(item_id, None)
 
     @classmethod
     def LoadDefaults(cls):
@@ -309,15 +306,27 @@ class KeyBinder(object):
             if reader != -1:
                 util.Log("[keybinder][info] Loading KeyProfile: %s" % ppath)
                 for line in reader:
-                    parts = line.split('=', 1)
+                    parts = line.split(u'=', 1)
+                    # Check that the line was formatted properly
                     if len(parts) == 2:
+                        # Try to find the ID value
                         item_id = _GetValueFromStr(parts[0])
                         if item_id is not None:
-                            keydict[item_id] = [ part.strip()
-                                                 for part in parts[1].split('+')
-                                                 if len(part.strip()) ]
-                            if parts[1].strip().endswith('++'):
-                                keydict[item_id].append('+')
+                            tmp = [ part.strip()
+                                    for part in parts[1].split(u'+')
+                                    if len(part.strip()) ]
+
+                            # Do some checking if the binding is valid
+                            nctrl = len([key for key in tmp
+                                         if key not in (u'Ctrl', u'Alt', u'Shift')])
+                            if nctrl:
+                                keydict[item_id] = tmp
+                                if parts[1].strip().endswith(u'++'):
+                                    keydict[item_id].append(u'+')
+                            else:
+                                # Invalid key binding
+                                continue
+
                 reader.close()
                 KeyBinder.keyprofile = keydict
                 KeyBinder.cprofile = pname
@@ -341,7 +350,7 @@ class KeyBinder(object):
             if writer != -1:
                 itemlst = list()
                 for item in KeyBinder.keyprofile.keys():
-                    itemlst.append("%s=%s%s" % (_FindStringRep(item),
+                    itemlst.append(u"%s=%s%s" % (_FindStringRep(item),
                                                 self.GetBinding(item).lstrip(),
                                                 os.linesep))
                 writer.writelines(sorted(itemlst))
@@ -356,18 +365,17 @@ class KeyBinder(object):
         @param keys: string or list of key strings
 
         """
-        if cls.cprofile is None:
-            util.Log("[keybinder][warn] No keyprofile has been loaded yet")
-        else:
-            if isinstance(keys, basestring):
-                keys = [ key.strip() for key in keys.split('+') ]
+        if isinstance(keys, basestring):
+            keys = [ key.strip() for key in keys.split('+') ]
 
-            if len(keys):
-                cls.keyprofile[item_id] = keys
-            elif cls.keyprofile.has_key(item_id):
-                del cls.keyprofile[item_id]
-            else:
-                pass
+        if len(keys):
+            # Set the binding
+            cls.keyprofile[item_id] = keys
+        elif item_id in cls.keyprofile:
+            # Clear the binding
+            del cls.keyprofile[item_id]
+        else:
+            pass
 
     @classmethod
     def SetProfileName(cls, pname):
@@ -460,17 +468,17 @@ class EdMenuBar(wx.MenuBar):
                         EdMenuBar.keybinder.GetBinding(ed_glob.ID_OPEN),
                         _("Open"))
         ## Setup File History in the File Menu
-        filemenu.AppendMenu(ed_glob.ID_FHIST, _("Open Recent"),
+        filemenu.AppendMenu(ed_glob.ID_FHIST, _("Open &Recent"),
                             filehist, _("Recently Opened Files"))
         filemenu.AppendSeparator()
-        filemenu.Append(ed_glob.ID_CLOSE, _("Close Page") + \
+        filemenu.Append(ed_glob.ID_CLOSE, _("&Close Tab") + \
                         EdMenuBar.keybinder.GetBinding(ed_glob.ID_CLOSE),
-                        _("Close Current Page"))
+                        _("Close Current Tab"))
         filemenu.Append(ed_glob.ID_CLOSE_WINDOW,
                         _("Close Window") + \
                         EdMenuBar.keybinder.GetBinding(ed_glob.ID_CLOSE_WINDOW),
                         _("Close the current window"))
-        filemenu.Append(ed_glob.ID_CLOSEALL, _("Close All Pages") + \
+        filemenu.Append(ed_glob.ID_CLOSEALL, _("Close All Tabs") + \
                         EdMenuBar.keybinder.GetBinding(ed_glob.ID_CLOSEALL),
                         _("Close all open tabs"))
         filemenu.AppendSeparator()
@@ -484,6 +492,13 @@ class EdMenuBar(wx.MenuBar):
                         EdMenuBar.keybinder.GetBinding(ed_glob.ID_SAVEALL),
                         _("Save all open pages"))
         filemenu.AppendSeparator()
+        filemenu.Append(ed_glob.ID_REVERT_FILE, _("Revert to Saved") + \
+                        EdMenuBar.keybinder.GetBinding(ed_glob.ID_REVERT_FILE),
+                        _("Revert file to last save point"))
+        filemenu.Append(ed_glob.ID_RELOAD_ENC, _("Reload with Encoding") + \
+                        u"..." + EdMenuBar.keybinder.GetBinding(ed_glob.ID_RELOAD_ENC),
+                        _("Reload the file with a specified encoding"))
+        filemenu.AppendSeparator()
         pmenu = EdMenu()
         pmenu.Append(ed_glob.ID_SAVE_PROFILE, _("Save Profile") + \
                      EdMenuBar.keybinder.GetBinding(ed_glob.ID_SAVE_PROFILE),
@@ -534,10 +549,16 @@ class EdMenuBar(wx.MenuBar):
         editmenu.Append(ed_glob.ID_PASTE, _("&Paste") + \
                         EdMenuBar.keybinder.GetBinding(ed_glob.ID_PASTE),
                         _("Paste Text from Clipboard to File"))
+        editmenu.Append(ed_glob.ID_PASTE_AFTER, _("P&aste After") + \
+                        EdMenuBar.keybinder.GetBinding(ed_glob.ID_PASTE_AFTER),
+                        _("Paste Text from Clipboard to File after the cursor"))
         editmenu.AppendSeparator()
         editmenu.Append(ed_glob.ID_SELECTALL, _("Select &All") + \
                         EdMenuBar.keybinder.GetBinding(ed_glob.ID_SELECTALL),
                         _("Select All Text in Document"))
+        editmenu.Append(ed_glob.ID_COLUMN_MODE, _("Column Mode") + \
+                        EdMenuBar.keybinder.GetBinding(ed_glob.ID_COLUMN_MODE),
+                        _("Enable column edit mode."), wx.ITEM_CHECK)
         editmenu.AppendSeparator()
         linemenu = EdMenu()
         linemenu.Append(ed_glob.ID_LINE_AFTER, _("New Line After") + \
@@ -586,6 +607,14 @@ class EdMenuBar(wx.MenuBar):
         editmenu.Append(ed_glob.ID_QUICK_FIND, _("&Quick Find") + \
                         EdMenuBar.keybinder.GetBinding(ed_glob.ID_QUICK_FIND),
                         _("Open the Quick Find Bar"))
+        editmenu.Append(ed_glob.ID_FIND_PREVIOUS, _("Find Previous") + \
+                        EdMenuBar.keybinder.GetBinding(ed_glob.ID_FIND_PREVIOUS),
+                        _("Goto previous match"))
+        editmenu.Append(ed_glob.ID_FIND_NEXT, _("Find Next") + \
+                        EdMenuBar.keybinder.GetBinding(ed_glob.ID_FIND_NEXT),
+                        _("Goto the next match"))
+        editmenu.Append(ed_glob.ID_FIND_SELECTED, _("Find Selected") + \
+                        EdMenuBar.keybinder.GetBinding(ed_glob.ID_FIND_SELECTED))
         editmenu.AppendSeparator()
         editmenu.Append(ed_glob.ID_PREF, _("Pr&eferences") + \
                         EdMenuBar.keybinder.GetBinding(ed_glob.ID_PREF),
@@ -635,6 +664,23 @@ class EdMenuBar(wx.MenuBar):
                         _("Show Whitespace Markers"), wx.ITEM_CHECK)
         viewmenu.AppendSubMenu(self._menus['viewedit'], _("Editor"), \
                                _("Toggle Editor View Options"))
+        viewfold = self._menus['viewfold'] = EdMenu()
+        viewfold.Append(ed_glob.ID_TOGGLE_FOLD, _("Toggle fold") + \
+                        EdMenuBar.keybinder.GetBinding(ed_glob.ID_TOGGLE_FOLD),
+                        _("Toggle current fold"))
+        viewfold.Append(ed_glob.ID_TOGGLE_ALL_FOLDS, _("Toggle all folds") + \
+                        EdMenuBar.keybinder.GetBinding(ed_glob.ID_TOGGLE_ALL_FOLDS),
+                        _("Toggle all folds"))
+        viewmenu.AppendSubMenu(self._menus['viewfold'], _("Code Folding"), \
+                               _("Code folding toggle actions"))
+
+        viewmenu.AppendSeparator()
+        viewmenu.Append(ed_glob.ID_PANELIST, _("Pane Navigator") + \
+                        EdMenuBar.keybinder.GetBinding(ed_glob.ID_PANELIST),
+                        _("View pane selection list"))
+        viewmenu.Append(ed_glob.ID_MAXIMIZE_EDITOR, _("Maximize Editor") + \
+                        EdMenuBar.keybinder.GetBinding(ed_glob.ID_MAXIMIZE_EDITOR),
+                        _("Toggle Editor Maximization"))
         viewmenu.AppendSeparator()
         viewmenu.Append(ed_glob.ID_GOTO_LINE, _("&Goto Line") + \
                         EdMenuBar.keybinder.GetBinding(ed_glob.ID_GOTO_LINE),
@@ -642,6 +688,7 @@ class EdMenuBar(wx.MenuBar):
         viewmenu.Append(ed_glob.ID_GOTO_MBRACE, _("Goto Matching Brace") + \
                         EdMenuBar.keybinder.GetBinding(ed_glob.ID_GOTO_MBRACE),
                         _("Move caret matching brace"))
+        viewmenu.AppendSeparator()
         viewmenu.Append(ed_glob.ID_NEXT_MARK, _("Next Bookmark") + \
                         EdMenuBar.keybinder.GetBinding(ed_glob.ID_NEXT_MARK),
                         _("View Line of Next Bookmark"))
@@ -670,12 +717,9 @@ class EdMenuBar(wx.MenuBar):
                           EdMenuBar.keybinder.GetBinding(ed_glob.ID_FONT),
                           _("Change Font Settings"))
         formatmenu.AppendSeparator()
-        formatmenu.Append(ed_glob.ID_COMMENT, _("Comment Lines") + \
-                          EdMenuBar.keybinder.GetBinding(ed_glob.ID_COMMENT),
-                          _("Comment the selected lines"))
-        formatmenu.Append(ed_glob.ID_UNCOMMENT, _("Uncomment Lines") + \
-                          EdMenuBar.keybinder.GetBinding(ed_glob.ID_UNCOMMENT),
-                          _("Uncomment the selected lines"))
+        formatmenu.Append(ed_glob.ID_TOGGLECOMMENT, _("Toggle Comment") + \
+                          EdMenuBar.keybinder.GetBinding(ed_glob.ID_TOGGLECOMMENT),
+                          _("Toggle comment on the selected line(s)"))
         formatmenu.AppendSeparator()
 
 
@@ -719,18 +763,18 @@ class EdMenuBar(wx.MenuBar):
 
         # Line EOL formatting submenu
         lineformat = self._menus['lineformat'] = EdMenu()
-        lineformat.Append(ed_glob.ID_EOL_MAC, _("Macintosh (\\r)") + \
+        lineformat.Append(ed_glob.ID_EOL_MAC, _("Old Macintosh (\\r)") + \
                           EdMenuBar.keybinder.GetBinding(ed_glob.ID_EOL_MAC),
                           _("Format all EOL characters to %s Mode") % \
-                          u"Macintosh (\\r)", wx.ITEM_CHECK)
+                          _(u"Old Macintosh (\\r)"), wx.ITEM_CHECK)
         lineformat.Append(ed_glob.ID_EOL_UNIX, _("Unix (\\n)") + \
                           EdMenuBar.keybinder.GetBinding(ed_glob.ID_EOL_UNIX),
                           _("Format all EOL characters to %s Mode") % \
-                          u"Unix (\\n)", wx.ITEM_CHECK)
+                          _(u"Unix (\\n)"), wx.ITEM_CHECK)
         lineformat.Append(ed_glob.ID_EOL_WIN, _("Windows (\\r\\n)") + \
                           EdMenuBar.keybinder.GetBinding(ed_glob.ID_EOL_WIN),
                           _("Format all EOL characters to %s Mode") % \
-                          "Windows (\\r\\n)", wx.ITEM_CHECK)
+                          _("Windows (\\r\\n)"), wx.ITEM_CHECK)
         formatmenu.AppendMenu(ed_glob.ID_EOL_MODE, _("EOL Mode"), lineformat,
                               _("End of line character formatting"))
 
@@ -882,7 +926,24 @@ class EdMenuBar(wx.MenuBar):
         """Reset all key bindings based on current binder profile"""
         for menu in self.GetMenus():
             for item in IterateMenuItems(menu[0]):
-                binding = EdMenuBar.keybinder.GetBinding(item.GetId())
+                item_id = item.GetId()
+                binding = EdMenuBar.keybinder.GetBinding(item_id)
+                if not len(binding):
+                    continue
+
+                # Verify binding and clear invalid ones from binder
+                tmp = [key.title() for key in binding.strip().split(u'+')]
+                nctrl = len([key for key in tmp
+                            if key not in ('Ctrl', 'Alt', 'Shift')])
+                if len(tmp) > 3 or not nctrl:
+                    EdMenuBar.keybinder.SetBinding(item_id, '')
+                    continue
+
+                # Reset the binding in the binder to ensure it is
+                # correctly formatted.
+                binding = u"\t" + u"+".join(tmp)
+                EdMenuBar.keybinder.SetBinding(item_id, binding)
+
                 clbl = item.GetText()
                 # Update the item if the shortcut has changed
                 if ('\t' in clbl and not clbl.endswith(binding)) or \
@@ -922,67 +983,82 @@ class EdMenuBar(wx.MenuBar):
 #---- Private Objects/Functions ----#
 
 _DEFAULT_BINDING = { # File Menu
-                     ed_glob.ID_NEW : ("Ctrl", "N"),
-                     ed_glob.ID_NEW_WINDOW : ("Ctrl", "Shift", "N"),
-                     ed_glob.ID_OPEN : ("Ctrl", "O"),
-                     ed_glob.ID_CLOSE : ("Ctrl", "W"),
-                     ed_glob.ID_CLOSE_WINDOW : ("Ctrl", "Shift", "W"),
-                     ed_glob.ID_SAVE : ("Ctrl", "S"),
-                     ed_glob.ID_SAVEAS : ("Ctrl", "Shift", "S"),
-                     ed_glob.ID_PRINT_SU : ("Ctrl", "Shift", "P"),
-                     ed_glob.ID_PRINT : ("Ctrl", "P"),
-                     ed_glob.ID_EXIT : ("Ctrl", "Q"),
+                     ed_glob.ID_NEW : (u"Ctrl", u"N"),
+                     ed_glob.ID_NEW_WINDOW : (u"Ctrl", u"Shift", u"N"),
+                     ed_glob.ID_OPEN : (u"Ctrl", u"O"),
+                     ed_glob.ID_CLOSE : (u"Ctrl", u"W"),
+                     ed_glob.ID_CLOSE_WINDOW : (u"Ctrl", u"Shift", u"W"),
+                     ed_glob.ID_SAVE : (u"Ctrl", u"S"),
+                     ed_glob.ID_SAVEAS : (u"Ctrl", u"Shift", u"S"),
+                     ed_glob.ID_PRINT_SU : (u"Ctrl", u"Shift", u"P"),
+                     ed_glob.ID_PRINT : (u"Ctrl", u"P"),
+                     ed_glob.ID_EXIT : (u"Ctrl", u"Q"),
 
                      # Edit Menu
-                     ed_glob.ID_UNDO : ("Ctrl", "Z"),
-                     ed_glob.ID_REDO : ("Ctrl", "Shift", "Z"),
-                     ed_glob.ID_CUT : ("Ctrl", "X"),
-                     ed_glob.ID_COPY : ("Ctrl", "C"),
-                     ed_glob.ID_PASTE : ("Ctrl", "V"),
-                     ed_glob.ID_SELECTALL : ("Ctrl", "A"),
-                     ed_glob.ID_LINE_AFTER : ("Ctrl", "L"),
-                     ed_glob.ID_LINE_BEFORE : ("Ctrl", "Shift", "L"),
-                     ed_glob.ID_CUT_LINE : ("Ctrl", "D"),
-                     ed_glob.ID_COPY_LINE : ("Ctrl", "Y"),
-                     ed_glob.ID_DUP_LINE : ("Ctrl", "Shift", "C"),
-                     ed_glob.ID_JOIN_LINES : ("Ctrl", "J"),
-                     ed_glob.ID_TRANSPOSE : ("Ctrl", "T"),
-                     ed_glob.ID_ADD_BM : ("Ctrl", "B"),
-                     ed_glob.ID_FIND : ("Ctrl", "Shift", "F"),
-                     ed_glob.ID_FIND_REPLACE : ("Ctrl", "R"),
-                     ed_glob.ID_QUICK_FIND : ("Ctrl", "F"),
+                     ed_glob.ID_UNDO : (u"Ctrl", u"Z"),
+                     ed_glob.ID_REDO : (u"Ctrl", u"Shift", u"Z"),
+                     ed_glob.ID_CUT : (u"Ctrl", u"X"),
+                     ed_glob.ID_COPY : (u"Ctrl", u"C"),
+                     ed_glob.ID_PASTE : (u"Ctrl", u"V"),
+                     ed_glob.ID_PASTE_AFTER : (u"Ctrl", u"Shift", u"V"),
+                     ed_glob.ID_SELECTALL : (u"Ctrl", u"A"),
+                     ed_glob.ID_COLUMN_MODE : (u"Ctrl", u"Shift", u"|"),
+                     ed_glob.ID_LINE_AFTER : (u"Ctrl", u"L"),
+                     ed_glob.ID_LINE_BEFORE : (u"Ctrl", u"Shift", u"L"),
+                     ed_glob.ID_CUT_LINE : (u"Ctrl", u"D"),
+                     ed_glob.ID_COPY_LINE : (u"Ctrl", u"Y"),
+                     ed_glob.ID_DUP_LINE : (u"Ctrl", u"Shift", u"C"),
+                     ed_glob.ID_JOIN_LINES : (u"Ctrl", u"J"),
+                     ed_glob.ID_TRANSPOSE : (u"Ctrl", u"T"),
+                     ed_glob.ID_ADD_BM : (u"Ctrl", u"B"),
+                     ed_glob.ID_FIND : (u"Ctrl", u"Shift", u"F"),
+                     ed_glob.ID_FIND_PREVIOUS : (u"Shift", u"F3"),
+                     ed_glob.ID_FIND_NEXT : (u"F3",),
+                     ed_glob.ID_FIND_REPLACE : (u"Ctrl", u"R"),
+                     ed_glob.ID_QUICK_FIND : (u"Ctrl", u"F"),
+                     ed_glob.ID_FIND_SELECTED : (u"Ctrl", u"F3"),
 
                      # View Menu
-                     ed_glob.ID_ZOOM_IN : ("Ctrl", "+"),
-                     ed_glob.ID_ZOOM_OUT : ("Ctrl", "-"),
-                     ed_glob.ID_ZOOM_NORMAL : ("Ctrl", "0"),
-                     ed_glob.ID_GOTO_LINE : ("Ctrl", "G"),
-                     ed_glob.ID_GOTO_MBRACE : ("Ctrl", "Shift", "B"),
-                     ed_glob.ID_NEXT_MARK : ("Alt", "Right"), # Win/Linux
-                     ed_glob.ID_PRE_MARK : ("Alt", "Left"), # Win/Linux
+                     ed_glob.ID_ZOOM_IN : (u"Ctrl", u"+"),
+                     ed_glob.ID_ZOOM_OUT : (u"Ctrl", u"-"),
+                     ed_glob.ID_ZOOM_NORMAL : (u"Ctrl", u"0"),
+                     ed_glob.ID_GOTO_LINE : (u"Ctrl", u"G"),
+                     ed_glob.ID_GOTO_MBRACE : (u"Ctrl", u"Shift", u"B"),
+                     ed_glob.ID_TOGGLE_FOLD : (u"Ctrl", u"Shift", u"T"),
+                     ed_glob.ID_NEXT_MARK : (u"Alt", u"Right"), # Win/Linux
+                     ed_glob.ID_PRE_MARK : (u"Alt", u"Left"), # Win/Linux
+                     ed_glob.ID_SHOW_SHELF : (u"Ctrl", u"Alt", u"S"),
+                     ed_glob.ID_PANELIST : (u"Alt", u"1"), # Win/Linux
+                     ed_glob.ID_MAXIMIZE_EDITOR : (u"Ctrl", u"M"), 
 
                      # Format Menu
-                     ed_glob.ID_COMMENT : ("Ctrl", "1"),
-                     ed_glob.ID_UNCOMMENT : ("Ctrl", "2"),
-                     ed_glob.ID_INDENT : ("Tab",),
-                     ed_glob.ID_UNINDENT : ("Shift", "Tab"),
-                     ed_glob.ID_USE_SOFTTABS : ("Ctrl", "Shift", "I"),
+                     ed_glob.ID_TOGGLECOMMENT : (u"Ctrl", u"1"),
+                     ed_glob.ID_INDENT : (u"Tab",),
+                     ed_glob.ID_UNINDENT : (u"Shift", u"Tab"),
+                     ed_glob.ID_USE_SOFTTABS : (u"Ctrl", u"Shift", u"I"),
 
                      # Tools Menu
-                     ed_glob.ID_COMMAND : ("Ctrl", "E"),
-                     ed_glob.ID_KWHELPER : ("Ctrl", "K"),
-                     ed_glob.ID_RUN_LAUNCH : ("F5",)
+                     ed_glob.ID_COMMAND : (u"Ctrl", u"E"),
+                     ed_glob.ID_KWHELPER : (u"Ctrl", u"K"),
+                     ed_glob.ID_RUN_LAUNCH : (u"F5",),
+                     ed_glob.ID_LAUNCH_LAST : (u"Shift", u"F5")
                      }
 
 # Set some platform specific keybindigs
 if wx.Platform == '__WXMAC__':
-    _DEFAULT_BINDING[ed_glob.ID_NEXT_MARK] = ("Ctrl", "Right")
-    _DEFAULT_BINDING[ed_glob.ID_PRE_MARK] = ("Ctrl", "Left")
+    _DEFAULT_BINDING[ed_glob.ID_NEXT_MARK] = (u"Ctrl", u"Down")
+    _DEFAULT_BINDING[ed_glob.ID_PRE_MARK] = (u"Ctrl", u"Up")
+    _DEFAULT_BINDING[ed_glob.ID_FIND_PREVIOUS] = (u"Ctrl", u"Shift", u"G")
+    _DEFAULT_BINDING[ed_glob.ID_FIND_NEXT] = (u"Ctrl", u"G")
+    _DEFAULT_BINDING[ed_glob.ID_GOTO_LINE] = (u"Ctrl", u"Shift", u"E")
+    _DEFAULT_BINDING[ed_glob.ID_PANELIST] = (u"Alt", u"Tab")
+    _DEFAULT_BINDING[ed_glob.ID_MAXIMIZE_EDITOR] = (u"Alt", u"M")
+    _DEFAULT_BINDING[ed_glob.ID_FIND_SELECTED] = (u"Ctrl", u"3")
 elif wx.Platform == '__WXMSW__':
      # FIXME: On Windows if Tab is bound to a menu item it is no longer
      #        usable elsewhere such as in the stc control. On Mac/Gtk there
      #        are not problems with it.
-    _DEFAULT_BINDING[ed_glob.ID_INDENT] = ("",)
+    _DEFAULT_BINDING[ed_glob.ID_INDENT] = (u"",)
 else:
     pass
 
@@ -1048,7 +1124,7 @@ def WalkMenu(menu, label, collection):
     @return: dict {menulabel : [menu id, (item1 id, label1),]}
 
     """
-    if not collection.has_key(label):
+    if label not in collection:
         collection[label] = list()
 
     for item in menu.GetMenuItems():
@@ -1056,7 +1132,7 @@ def WalkMenu(menu, label, collection):
         if item.IsSubMenu():
             # Ignore dynamically generated menus
             if i_id not in (ed_glob.ID_FHIST, ed_glob.ID_LEXER,
-                            ed_glob.ID_SHELF, ed_glob.ID_PERSPECTIVES):
+                             ed_glob.ID_PERSPECTIVES):
                 ilbl = item.GetItemLabelText()
                 collection[ilbl] = [i_id, ]
                 WalkMenu(item.GetSubMenu(), ilbl, collection)
