@@ -13,8 +13,8 @@ Provides a dialog for downloading, installing and configuring plugins or Editra.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__cvsid__ = "$Id: plugdlg.py 52890 2008-03-28 02:38:56Z CJP $"
-__revision__ = "$Revision: 52890 $"
+__cvsid__ = "$Id: plugdlg.py 57350 2008-12-15 04:25:13Z CJP $"
+__revision__ = "$Revision: 57350 $"
 
 #--------------------------------------------------------------------------#
 # Dependancies
@@ -54,11 +54,11 @@ def MakeThemeTool(tool_id):
     @todo: why does drawing a bitmap overlay on gtk not draw on transparent area
 
     """
-    osize = Profile_Get('ICON_SZ', 'size_tuple', (24, 24))
+    osize = Profile_Get('ICON_SZ', default=(24, 24))
     Profile_Set('ICON_SZ', (32, 32))
     base = wx.ArtProvider.GetBitmap(str(tool_id), wx.ART_TOOLBAR)
     Profile_Set('ICON_SZ', osize)
-    if not base.IsOk() or base.GetSize() != (32, 32):
+    if not base.IsOk():
         base = wx.ArtProvider.GetBitmap(wx.ART_WARNING,
                                         wx.ART_TOOLBAR,
                                         size=(32, 32))
@@ -524,7 +524,7 @@ class DownloadPanel(wx.Panel):
             else:
                 self._downlb.Disable()
 
-            if self._dl_list.has_key(pin):
+            if pin in self._dl_list:
                 del self._dl_list[pin]
 
     def PopulateList(self):
@@ -557,14 +557,14 @@ class DownloadPanel(wx.Panel):
 
         """
         # Removed downloaded data
-        if self._eggbasket.has_key(item):
+        if item in self._eggbasket:
             del self._eggbasket[item]
 
         # Remove download entry data
         match = self.EGG_PATTERN(item)
         if match:
             plugin_name = match.group('name').lower()
-            if self._dl_list.has_key(plugin_name):
+            if plugin_name in self._dl_list:
                 del self._dl_list[plugin_name]
 
     def UpdateList(self, url=PLUGIN_REPO):
