@@ -16,13 +16,15 @@ and setting of the program by setting values in the Profile.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: prefdlg.py 54033 2008-06-08 22:17:13Z CJP $"
-__revision__ = "$Revision: 54033 $"
+__svnid__ = "$Id: prefdlg.py 58818 2009-02-09 21:04:29Z CJP $"
+__revision__ = "$Revision: 58818 $"
 
 #----------------------------------------------------------------------------#
 # Dependancies
 import wx
 import wx.lib.mixins.listctrl as listmix
+import locale
+import encodings
 import os
 import sys
 
@@ -33,11 +35,13 @@ from profiler import Profile_Get, Profile_Set
 import ed_i18n
 import ed_event
 import ed_crypt
+import ed_stc
 import updater
 import util
 import syntax.syntax as syntax
 import ed_msg
 import eclib.elistmix as elistmix
+import eclib.ecpickers as ecpickers
 
 #----------------------------------------------------------------------------#
 # Globals
@@ -48,20 +52,20 @@ ID_UPDATE_MSG   = wx.NewId()
 _ = wx.GetTranslation
 #----------------------------------------------------------------------------#
 # Class Globals
-from extern.embeddedimage import PyEmbeddedImage
+#from extern.embeddedimage import PyEmbeddedImage
 
-ButtonBackground = PyEmbeddedImage(
-    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABHNCSVQICAgIfAhkiAAAAfhJ"
-    "REFUWIXtlz9uGkEUh7/ZWbDdEAWH0ltRYCKRigNQxhFS3EUuUjlGspQLODfIAVxEkVPFzgmc"
-    "LhegCoolupWQ6GIcWIvE7DwmBX9iwGBiMG74SVvM6M37ffPeaEarGFQceJbJZA611inmKBEp"
-    "l0qlfeA7UOvNq2sxu6lUKpfNZne2nm+x/mR9nv6c/zzn9OspxWLxuFwuvwakB+BEIpE3hb3C"
-    "+2QyGfM8b67Gw6pUKhy8O/jQbDb3AVFAbvvl9rd8Pn+vxtdVrVY5+nR07Pv+W3djw9vzPI8g"
-    "uATsQgBisUdsbqZ3fN//6EZc91UikSAIGrcutNZirUUphVLq1vhJeppOc3b249A1xlD/VZ9s"
-    "3K2MYjbTkbxtm3KNGBpT7H58FrgrlxGDKyI0GsHdAWaQiNBpQX2wBRM3ZKcJmry0J2MMrhjh"
-    "MnigChjBDY2hVrtAaz3zyZ5W1lpEhNAYFN3KxB/HiUai/SCtNdHoylwMW60rROTfOGxRu+g8"
-    "B32AYWmtWYmuDswppVhbXcNxnJH4tm0ThiHGhNihjFetPwMAAznHAYyTqyNA50IaosNaS7t9"
-    "s9E4/TfAvDVayyXAEmAJsARYAixWLxzgywMC/AbI0XmQFv19pvMvigMUgPoCzevA7nA5CgsE"
-    "KNzUD6fbjpN7ND7pevQP/18X4iAGbqux9wAAAABJRU5ErkJggg==")
+#ButtonBackground = PyEmbeddedImage(
+#    "iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABHNCSVQICAgIfAhkiAAAAfhJ"
+#    "REFUWIXtlz9uGkEUh7/ZWbDdEAWH0ltRYCKRigNQxhFS3EUuUjlGspQLODfIAVxEkVPFzgmc"
+#    "LhegCoolupWQ6GIcWIvE7DwmBX9iwGBiMG74SVvM6M37ffPeaEarGFQceJbJZA611inmKBEp"
+#    "l0qlfeA7UOvNq2sxu6lUKpfNZne2nm+x/mR9nv6c/zzn9OspxWLxuFwuvwakB+BEIpE3hb3C"
+#    "+2QyGfM8b67Gw6pUKhy8O/jQbDb3AVFAbvvl9rd8Pn+vxtdVrVY5+nR07Pv+W3djw9vzPI8g"
+#    "uATsQgBisUdsbqZ3fN//6EZc91UikSAIGrcutNZirUUphVLq1vhJeppOc3b249A1xlD/VZ9s"
+#    "3K2MYjbTkbxtm3KNGBpT7H58FrgrlxGDKyI0GsHdAWaQiNBpQX2wBRM3ZKcJmry0J2MMrhjh"
+#    "MnigChjBDY2hVrtAaz3zyZ5W1lpEhNAYFN3KxB/HiUai/SCtNdHoylwMW60rROTfOGxRu+g8"
+#    "B32AYWmtWYmuDswppVhbXcNxnJH4tm0ThiHGhNihjFetPwMAAznHAYyTqyNA50IaosNaS7t9"
+#    "s9E4/TfAvDVayyXAEmAJsARYAixWLxzgywMC/AbI0XmQFv19pvMvigMUgPoCzevA7nA5CgsE"
+#    "KNzUD6fbjpN7ND7pevQP/18X4iAGbqux9wAAAABJRU5ErkJggg==")
 
 def MakeThemeTool(tool_id):
     """Makes a themed bitmap for the tool book of the pref dialog.
@@ -70,21 +74,26 @@ def MakeThemeTool(tool_id):
 
     """
     osize = Profile_Get('ICON_SZ', 'size_tuple', (24, 24))
-    Profile_Set('ICON_SZ', (24, 24))
+    Profile_Set('ICON_SZ', (32, 32))
     over = wx.ArtProvider.GetBitmap(str(tool_id), wx.ART_TOOLBAR)
     Profile_Set('ICON_SZ', osize)
-    mdc = wx.MemoryDC(ButtonBackground.GetBitmap())
-    if over.IsOk():
-        # Create overlay
-        if over.GetSize() != (24, 24):
-            over = over.ConvertToImage()
-            over.Rescale(24, 24, wx.IMAGE_QUALITY_HIGH)
-            over = over.ConvertToBitmap()
+    return over
 
-        # Draw overlay onto button
-        mdc.SetBrush(wx.TRANSPARENT_BRUSH)
-        mdc.DrawBitmap(over, 4, 4, True)
-    return mdc.GetAsBitmap()
+#----------------------------------------------------------------------------#
+
+class PreferencesPanelBase(object):
+    def __init__(self):
+        object.__init__(self)
+
+        # Attributes
+        self._layout_done = False
+
+    def DoSelected(self):
+        """Handle initial selection to create controls"""
+        if not self._layout_done:
+            self._layout_done = True
+            self._DoLayout()
+            self.SetAutoLayout(True)
 
 #----------------------------------------------------------------------------#
 
@@ -110,6 +119,8 @@ class PreferencesDialog(wx.Frame):
         self.SetTransparent(Profile_Get('ALPHA', 'int', 255))
 
         # Attributes
+        self._accel = wx.AcceleratorTable([(wx.ACCEL_NORMAL, wx.WXK_ESCAPE, wx.ID_CLOSE)])
+        self.SetAcceleratorTable(self._accel)
         self._tbook = PrefTools(self)
         sizer = wx.BoxSizer(wx.VERTICAL)
 
@@ -121,6 +132,7 @@ class PreferencesDialog(wx.Frame):
         wx.GetApp().RegisterWindow(repr(self), self, True)
 
         # Bind Events
+        self.Bind(wx.EVT_MENU, lambda evt: self.Close(), id=wx.ID_CLOSE)
         self.Bind(wx.EVT_CLOSE, self.OnClose)
         self.Bind(wx.EVT_SHOW, self.OnShow)
 
@@ -198,8 +210,16 @@ class PrefTools(wx.Toolbook):
 
         # Event Handlers
         self.Bind(wx.EVT_TOOLBOOK_PAGE_CHANGED, self.OnPageChanged)
+        self.Bind(wx.EVT_TOOLBOOK_PAGE_CHANGING, self.OnPageChanging)
         self.Bind(wx.EVT_PAINT, self.OnPaint)
 
+    def OnPageChanging(self, evt):
+        sel = evt.GetSelection()
+        page = self.GetPage(sel)
+        if hasattr(page, 'DoSelected'):
+            page.DoSelected()
+        evt.Skip()
+
     def OnPageChanged(self, evt=None):
         """Resizes the dialog based on the pages size
         @todo: animate the resizing so its smoother
@@ -219,9 +239,12 @@ class PrefTools(wx.Toolbook):
             width = psz.GetWidth()
         else:
             width = tbw
+
+        page.Freeze()
         parent.SetClientSize((width, psz.GetHeight() + tbh))
         parent.SendSizeEvent()
         parent.Layout()
+        page.Thaw()
         if evt is not None:
             evt.Skip()
 
@@ -257,53 +280,7 @@ class PrefTools(wx.Toolbook):
 
 #-----------------------------------------------------------------------------#
 
-class PrefPanelBase(wx.Panel):
-    """Base of all preference panels
-    @summary: Provides a panel with a painted background
-
-    """
-    def __init__(self, parent, style=wx.TAB_TRAVERSAL | wx.SUNKEN_BORDER):
-        """Default Constructor
-        @param parent: The panels parent
-
-        """
-        wx.Panel.__init__(self, parent, style=style)
-
-        # Event Handlers
-        if wx.Platform == '__WXMAC__':
-            self.Bind(wx.EVT_PAINT, self.OnPaint)
-
-    def OnPaint(self, evt):
-        """Paints the panels background
-        @param evt: Event that called this handler
-
-        """
-        # This is odd but I have recieved a number of error reports where
-        # wx is being reported as None in this function requires more
-        # investigation as I cannot reproduce
-        if 'wx' not in globals() or wx is None:
-            evt.Skip()
-            return
-
-        dc = wx.PaintDC(self)
-        gc = wx.GraphicsContext.Create(dc)
-        col1 = wx.SystemSettings_GetColour(wx.SYS_COLOUR_3DFACE)
-        brush = gc.CreateBrush(wx.Brush(util.AdjustColour(col1, 5)))
-        rect = self.GetRect()
-
-        # Create the background path
-        path = gc.CreatePath()
-        path.AddRectangle(0, 0, rect.width, rect.height)
-
-        gc.SetPen(wx.Pen(util.AdjustColour(col1, -10), 1))
-        gc.SetBrush(brush)
-        gc.DrawPath(path)
-
-        evt.Skip()
-
-#-----------------------------------------------------------------------------#
-
-class GeneralPanel(PrefPanelBase):
+class GeneralPanel(wx.Panel, PreferencesPanelBase):
     """Creates a panel with controls for Editra's general settings
     @summary: Panel with a number of controls that affect the users
               global profile setting for how Editra should operate.
@@ -314,21 +291,25 @@ class GeneralPanel(PrefPanelBase):
         @param parent: Parent window of this panel
 
         """
-        PrefPanelBase.__init__(self, parent)
+        wx.Panel.__init__(self, parent)
+        PreferencesPanelBase.__init__(self)
 
         # Attributes
         self.SetToolTipString(_("Changes made in this dialog are saved in your "
                                 "current profile. Some Items such as Language "
                                 "require the program to be restarted before "
                                 "taking effect."))
-        self.__DoLayout()
+
+        # Layout this page right away
+        self._DoLayout()
+        self._layout_done = True
 
         # Event Handlers
         self.Bind(wx.EVT_CHECKBOX, self.OnCheck)
         self.Bind(wx.EVT_CHOICE, self.OnChoice)
         self.Bind(wx.EVT_COMBOBOX, self.OnChoice)
 
-    def __DoLayout(self):
+    def _DoLayout(self):
         """Add the controls and do the layout
         @note: do not call this after __init__
 
@@ -338,7 +319,7 @@ class GeneralPanel(PrefPanelBase):
         msizer.AddMany([(wx.StaticText(self, label=_("Editor Mode") + u": "),
                          0, wx.ALIGN_CENTER_VERTICAL), ((5, 5), 0),
                         (ExChoice(self, ed_glob.ID_PREF_MODE,
-                                  choices=['CODE', 'DEBUG'],
+                                  choices=['CODE', 'DEBUG', 'VERBOSE DEBUG'],
                                   default=Profile_Get('MODE')),
                           0, wx.ALIGN_CENTER_VERTICAL)])
 
@@ -346,8 +327,7 @@ class GeneralPanel(PrefPanelBase):
         psizer.AddMany([(wx.StaticText(self, label=_("Printer Mode") + u": "),
                          0, wx.ALIGN_CENTER_VERTICAL), ((5, 5), 0),
                         (ExChoice(self, ed_glob.ID_PRINT_MODE,
-                                  choices=['Black/White', 'Colour/White',
-                                         'Colour/Default', 'Inverse', 'Normal'],
+                                  choices=GetPrintModeStrings(),
                                   default=Profile_Get('PRINT_MODE')),
                          0, wx.ALIGN_CENTER_VERTICAL)])
 
@@ -374,6 +354,24 @@ class GeneralPanel(PrefPanelBase):
                                    default=Profile_Get('FHIST_LVL', 'str')),
                          0, wx.ALIGN_CENTER_VERTICAL)])
 
+        # Encoding options
+        d_encoding = Profile_Get('ENCODING',
+                                 'str',
+                                 default=locale.getpreferredencoding())
+        if d_encoding is None:
+            d_encoding = 'utf-8'
+            Profile_Set('ENCODING', d_encoding)
+
+        d_encoding = encodings.normalize_encoding(d_encoding)
+
+        enc_ch = ExChoice(self, ed_glob.ID_PREF_ENCODING,
+                          choices=util.GetAllEncodings(),
+                          default=d_encoding)
+        enc_ch.SetToolTipString(_("Encoding to try when auto detection fails"))
+        enc_sz = wx.BoxSizer(wx.HORIZONTAL)
+        enc_sz.AddMany([(wx.StaticText(self, label=_("Prefered Encoding") + u":"),
+                        0, wx.ALIGN_CENTER_VERTICAL), ((5, 5),), (enc_ch, 1)])
+
         win_cb = wx.CheckBox(self, ed_glob.ID_NEW_WINDOW,
                              _("Open files in new windows by default"))
         win_cb.SetValue(Profile_Get('OPEN_NW'))
@@ -384,6 +382,9 @@ class GeneralPanel(PrefPanelBase):
                                 _("Check if on disk file has been "
                                   "modified by others"))
         chkmod_cb.SetValue(Profile_Get('CHECKMOD'))
+        eolwarn_cb = wx.CheckBox(self, ed_glob.ID_PREF_WARN_EOL,
+                                 _("Warn when mixed eol characters are detected"))
+        eolwarn_cb.SetValue(Profile_Get('WARN_EOL'))
 
         # Locale
         lsizer = wx.BoxSizer(wx.HORIZONTAL)
@@ -394,7 +395,7 @@ class GeneralPanel(PrefPanelBase):
                          0, wx.ALIGN_CENTER_VERTICAL)])
 
         # Layout items
-        sizer = wx.FlexGridSizer(15, 2, 5, 5)
+        sizer = wx.FlexGridSizer(18, 2, 5, 5)
         sizer.AddMany([((10, 10), 0), ((10, 10), 0),
                        (wx.StaticText(self,
                         label=_("Startup Settings") + u": "),
@@ -406,10 +407,12 @@ class GeneralPanel(PrefPanelBase):
                        ((5, 5),), (chk_update, 0),
                        ((5, 5),), ((5, 5),),
                        (wx.StaticText(self, label=_("File Settings") + u": "),
-                        0, wx.ALIGN_CENTER_VERTICAL), (fhsizer, 0),
+                        0, wx.ALIGN_CENTER_VERTICAL), (enc_sz, 0),
+                       ((5, 5),), (fhsizer, 0),
                        ((5, 5),), (win_cb, 0),
                        ((5, 5),), (pos_cb, 0),
                        ((5, 5),), (chkmod_cb, 0),
+                       ((5, 5),), (eolwarn_cb, 0),
                        ((5, 5),), ((5, 5),),
                        (wx.StaticText(self, label=_("Locale Settings") + u": "),
                         0, wx.ALIGN_CENTER_VERTICAL), (lsizer, 0),
@@ -427,9 +430,10 @@ class GeneralPanel(PrefPanelBase):
         util.Log("[prefdlg][evt] General Page: Check box clicked")
         e_id = evt.GetId()
         e_obj = evt.GetEventObject()
-        if e_id in [ed_glob.ID_APP_SPLASH, ed_glob.ID_PREF_SPOS,
+        if e_id in (ed_glob.ID_APP_SPLASH, ed_glob.ID_PREF_SPOS,
                     ed_glob.ID_PREF_CHKMOD, ed_glob.ID_SESSION,
-                    ed_glob.ID_NEW_WINDOW, ed_glob.ID_PREF_CHKUPDATE]:
+                    ed_glob.ID_NEW_WINDOW, ed_glob.ID_PREF_CHKUPDATE,
+                    ed_glob.ID_PREF_WARN_EOL):
             Profile_Set(ed_glob.ID_2_PROF[e_id], e_obj.GetValue())
         elif e_id == ed_glob.ID_REPORTER:
             Profile_Set(ed_glob.ID_2_PROF[e_id], not e_obj.GetValue())
@@ -444,20 +448,26 @@ class GeneralPanel(PrefPanelBase):
         @note: Also handles the Language ComboBox
 
         """
-        util.Log("[prefldg][evt] General Page: Choice event caught")
         e_id = evt.GetId()
         e_obj = evt.GetEventObject()
-        if e_id in [ed_glob.ID_PREF_MODE, ed_glob.ID_PRINT_MODE,
-                    ed_glob.ID_PREF_FHIST, ed_glob.ID_PREF_LANG]:
+        if e_id in [ed_glob.ID_PREF_MODE,
+                    ed_glob.ID_PREF_FHIST,
+                    ed_glob.ID_PREF_LANG,
+                    ed_glob.ID_PREF_ENCODING]:
+
             Profile_Set(ed_glob.ID_2_PROF[e_id], e_obj.GetValue())
             if e_id == ed_glob.ID_PREF_MODE:
-                ed_glob.DEBUG = ('DEBUG' in e_obj.GetValue())
+                val = e_obj.GetValue()
+                ed_glob.DEBUG = ('DEBUG' in val)
+                ed_glob.VDEBUG = ('VERBOSE' in val)
+        elif e_id == ed_glob.ID_PRINT_MODE:
+            Profile_Set(ed_glob.ID_2_PROF[e_id], e_obj.GetSelection())
         else:
             evt.Skip()
 
 #-----------------------------------------------------------------------------#
 
-class DocumentPanel(PrefPanelBase):
+class DocumentPanel(wx.Panel, PreferencesPanelBase):
     """Creates a panel with controls for Editra's editing settings
     @summary: Contains a wx.Notebook that contains a number of pages with
               setting controls for how documents are handled by the
@@ -469,11 +479,8 @@ class DocumentPanel(PrefPanelBase):
         @param parent: Parent window of this panel
 
         """
-        PrefPanelBase.__init__(self, parent)
-
-        # Attributes
-        self._DoLayout()
-        self.SetAutoLayout(True)
+        wx.Panel.__init__(self, parent)
+        PreferencesPanelBase.__init__(self)
 
     def _DoLayout(self):
         """Do the layout of the panel
@@ -490,6 +497,14 @@ class DocumentPanel(PrefPanelBase):
         msizer.AddMany([(sizer, 1, wx.EXPAND), ((10, 10), 0)])
         self.SetSizer(msizer)
 
+    def GetSize(self):
+        """Get the size of the panel
+        @return: wx.Size
+
+        """
+        sz = wx.Panel.GetSize(self)
+        return wx.Size(sz[0] + 35, sz[1])
+
 class DocGenPanel(wx.Panel):
     """Panel used for general document settings in the DocumentPanel's
     notebook.
@@ -508,15 +523,15 @@ class DocGenPanel(wx.Panel):
         wx.Panel.__init__(self, parent)
 
         # Layout
-        self.__DoLayout()
+        self._DoLayout()
         self.SetAutoLayout(True)
 
         # Event Handlers
         self.Bind(wx.EVT_CHECKBOX, self.OnUpdateEditor)
         self.Bind(wx.EVT_CHOICE, self.OnUpdateEditor)
-        self.Bind(ed_event.EVT_NOTIFY, self.OnFontChange)
+        self.Bind(ecpickers.EVT_FONT_CHANGED, self.OnFontChange)
 
-    def __DoLayout(self):
+    def _DoLayout(self):
         """Layout the controls
         @note: Do not call this after __init__
 
@@ -538,21 +553,28 @@ class DocGenPanel(wx.Panel):
                           default=Profile_Get('INDENTWIDTH', 'str')), 0,
                         wx.ALIGN_CENTER_VERTICAL)])
 
+        at_cb = wx.CheckBox(self, ed_glob.ID_PREF_AUTOTRIM,
+                            _("Automatically trim whitespace on save"))
+        at_cb.SetValue(Profile_Get('AUTO_TRIM_WS', 'bool', False))
         ut_cb = wx.CheckBox(self, ed_glob.ID_PREF_TABS,
                             _("Use Tabs Instead of Spaces"))
         ut_cb.SetValue(Profile_Get('USETABS', 'bool', False))
         bsu_cb = wx.CheckBox(self, ed_glob.ID_PREF_UNINDENT,
                              _("Backspace Unindents"))
         bsu_cb.SetValue(Profile_Get('BSUNINDENT', 'bool', True))
+
         eolsz = wx.BoxSizer(wx.HORIZONTAL)
+
+        # NOTE: order must be kept insync with vals in ed_glob, ed_stc
+        eolmode = ExChoice(self, ed_glob.ID_EOL_MODE,
+                           choices=[_("Old Macintosh (\\r)"), _("Unix (\\n)"),
+                                    _("Windows (\\r\\n)")])
+        eolmode.SetSelection(Profile_Get('EOL_MODE'))
+                        
         eolsz.AddMany([(wx.StaticText(self,
                         label=_("Default EOL Mode") + u": "),
                         0, wx.ALIGN_CENTER_VERTICAL), ((5, 5), 0),
-                       (ExChoice(self, ed_glob.ID_EOL_MODE,
-                                 choices=[_("Macintosh (\\r)"), _("Unix (\\n)"),
-                                          _("Windows (\\r\\n)")],
-                                 default=Profile_Get('EOL')),
-                        0, wx.ALIGN_CENTER_VERTICAL)])
+                       (eolmode, 0, wx.ALIGN_CENTER_VERTICAL)])
 
         # View Options
         aa_cb = wx.CheckBox(self, ed_glob.ID_PREF_AALIAS, _("AntiAliasing"))
@@ -571,21 +593,22 @@ class DocGenPanel(wx.Panel):
         fnt = Profile_Get('FONT1', 'font', wx.Font(10, wx.FONTFAMILY_MODERN,
                                                    wx.FONTSTYLE_NORMAL,
                                                    wx.FONTWEIGHT_NORMAL))
-        fpick = PyFontPicker(self, DocGenPanel.ID_FONT_PICKER, fnt)
+        fpick = ecpickers.PyFontPicker(self, DocGenPanel.ID_FONT_PICKER, fnt)
         fpick.SetToolTipString(_("Sets the main/default font of the document"))
         fnt = Profile_Get('FONT2', 'font', wx.Font(10, wx.FONTFAMILY_SWISS,
                                                    wx.FONTSTYLE_NORMAL,
                                                    wx.FONTWEIGHT_NORMAL))
-        fpick2 = PyFontPicker(self, DocGenPanel.ID_FONT_PICKER2, fnt)
+        fpick2 = ecpickers.PyFontPicker(self, DocGenPanel.ID_FONT_PICKER2, fnt)
         fpick2.SetToolTipString(_("Sets a secondary font used for special "
                                   "regions when syntax highlighting is in use"))
 
         # Layout
-        sizer = wx.FlexGridSizer(18, 2, 5, 5)
+        sizer = wx.FlexGridSizer(19, 2, 5, 5)
         sizer.AddGrowableCol(1, 1)
         sizer.AddMany([((10, 10), 0), ((10, 10), 0),
                        (wx.StaticText(self, label=_("Format") + u": "),
-                        0, wx.ALIGN_CENTER_VERTICAL), (ut_cb, 0),
+                        0, wx.ALIGN_CENTER_VERTICAL), (at_cb, 0),
+                       ((5, 5), 0), (ut_cb, 0),
                        ((5, 5), 0), (bsu_cb, 0),
                        ((5, 5), 0), (tabsz, 0),
                        ((5, 5), 0), (indentsz, 0),
@@ -611,7 +634,7 @@ class DocGenPanel(wx.Panel):
 
     @staticmethod
     def OnFontChange(evt):
-        """Handles L{ed_event.EVT_NOTIFY} from the font controls"""
+        """Handles L{ecpickers.EVT_FONT_CHANGED} from the font controls"""
         e_id = evt.GetId()
         if e_id in [DocGenPanel.ID_FONT_PICKER, DocGenPanel.ID_FONT_PICKER2]:
             font = evt.GetValue()
@@ -652,13 +675,22 @@ class DocGenPanel(wx.Panel):
                     ed_glob.ID_PREF_AALIAS, ed_glob.ID_SHOW_EOL,
                     ed_glob.ID_SHOW_LN, ed_glob.ID_SHOW_WS,
                     ed_glob.ID_WORD_WRAP, ed_glob.ID_PREF_AALIAS,
-                    ed_glob.ID_PREF_INDENTW]:
-            Profile_Set(ed_glob.ID_2_PROF[e_id],
-                        evt.GetEventObject().GetValue())
-            wx.CallLater(25, DoUpdates)
+                    ed_glob.ID_PREF_INDENTW, ed_glob.ID_PREF_AUTOTRIM ]:
+
+            e_value = evt.GetEventObject().GetValue()
+            if e_id == ed_glob.ID_EOL_MODE:
+                e_value = evt.GetEventObject().GetSelection()
+
+            Profile_Set(ed_glob.ID_2_PROF[e_id], e_value)
+
+            # Do updates for everything but text encoding
+            if e_id not in (ed_glob.ID_PREF_AUTOTRIM, ):
+                wx.CallLater(25, DoUpdates)
         else:
             evt.Skip()
 
+#----------------------------------------------------------------------------#
+
 class DocCodePanel(wx.Panel):
     """Panel used for programming settings
     @summary: Houses many of the controls for configuring the editors features
@@ -673,7 +705,7 @@ class DocCodePanel(wx.Panel):
         wx.Panel.__init__(self, parent)
 
         # Layout
-        self.__DoLayout()
+        self._DoLayout()
         self.SetAutoLayout(True)
 
         # Event Handlers
@@ -681,7 +713,7 @@ class DocCodePanel(wx.Panel):
         self.Bind(wx.EVT_CHOICE, self.OnCheck)
         self.Bind(wx.EVT_SPINCTRL, self.OnSpin)
 
-    def __DoLayout(self):
+    def _DoLayout(self):
         """Layout the page
         @note: Do not call this after __init__
 
@@ -748,8 +780,7 @@ class DocCodePanel(wx.Panel):
         msizer.AddMany([((10, 10), 0), (sizer, 1, wx.EXPAND), ((10, 10), 0)])
         self.SetSizer(msizer)
 
-    @staticmethod
-    def OnCheck(evt):
+    def OnCheck(self, evt):
         """Handles the events from this panels check boxes
         @param evt: wx.CommandEvent
 
@@ -760,8 +791,19 @@ class DocCodePanel(wx.Panel):
                     ed_glob.ID_AUTOCOMP, ed_glob.ID_AUTOINDENT,
                     ed_glob.ID_PREF_EDGE, ed_glob.ID_VI_MODE,
                     ed_glob.ID_PREF_DLEXER, ed_glob.ID_HLCARET_LINE):
-            Profile_Set(ed_glob.ID_2_PROF[e_id],
-                        evt.GetEventObject().GetValue())
+
+            e_val = evt.GetEventObject().GetValue()
+
+            # Update Profile
+            Profile_Set(ed_glob.ID_2_PROF[e_id], e_val)
+
+            # Make ui adjustments
+            if e_id == ed_glob.ID_SHOW_EDGE:
+                spin = self.FindWindowById(ed_glob.ID_PREF_EDGE)
+                if spin is not None:
+                    spin.Enable(e_val)
+
+            # Inform views of preference changes
             wx.CallLater(25, DoUpdates)
         else:
             evt.Skip()
@@ -783,6 +825,8 @@ class DocCodePanel(wx.Panel):
         else:
             evt.Skip()
 
+#----------------------------------------------------------------------------#
+
 class DocSyntaxPanel(wx.Panel):
     """Document syntax config panel
     @summary: Manages the configuration of the syntax highlighting
@@ -798,11 +842,10 @@ class DocSyntaxPanel(wx.Panel):
 
         # Attributes
         self._elist = ExtListCtrl(self)
-        self._elist.LoadList()
         self._elist.SetMinSize((425, 200))
 
         # Layout page
-        self.__DoLayout()
+        self._DoLayout()
         self.SetAutoLayout(True)
 
         # Event Handlers
@@ -810,7 +853,7 @@ class DocSyntaxPanel(wx.Panel):
         self.Bind(wx.EVT_CHECKBOX, self.OnSynChange)
         self.Bind(wx.EVT_CHOICE, self.OnSynChange)
 
-    def __DoLayout(self):
+    def _DoLayout(self):
         """Layout all the controls
         @note: Do not call this after __init__
 
@@ -893,7 +936,7 @@ class DocSyntaxPanel(wx.Panel):
 
 #-----------------------------------------------------------------------------#
 
-class AppearancePanel(PrefPanelBase):
+class AppearancePanel(wx.Panel, PreferencesPanelBase):
     """Creates a panel with controls for Editra's appearance settings
     @summary: contains all the controls for configuring the appearance
               related settings in Editra.
@@ -904,17 +947,15 @@ class AppearancePanel(PrefPanelBase):
         @param parent: Parent window of this panel
 
         """
-        PrefPanelBase.__init__(self, parent)
-
-        # Layout
-        self._DoLayout()
+        wx.Panel.__init__(self, parent)
+        PreferencesPanelBase.__init__(self)
 
         # Event Handlers
         self.Bind(wx.EVT_CHECKBOX, self.OnCheck)
         self.Bind(wx.EVT_CHOICE, self.OnChoice)
         self.Bind(wx.EVT_SLIDER, self.OnSetTransparent, \
                   id=ed_glob.ID_TRANSPARENCY)
-        self.Bind(ed_event.EVT_NOTIFY, self.OnFontChange)
+        self.Bind(ecpickers.EVT_FONT_CHANGED, self.OnFontChange)
 
     def _DoLayout(self):
         """Add and layout the widgets
@@ -936,6 +977,9 @@ class AppearancePanel(PrefPanelBase):
                            (ExChoice(self, ed_glob.ID_PREF_ICONSZ,
                                      ['16', '24', '32'],
                              str(Profile_Get('ICON_SZ', 'size_tuple')[0])), 0)])
+        tabicon_cb = wx.CheckBox(self, ed_glob.ID_PREF_TABICON,
+                                 _("Show Icons on Tabs"))
+        tabicon_cb.SetValue(Profile_Get('TABICONS'))
 
         # Layout Section
         mainw = wx.GetApp().GetActiveWindow()
@@ -962,20 +1006,16 @@ class AppearancePanel(PrefPanelBase):
 
         # Font
         fnt = Profile_Get('FONT3', 'font', wx.NORMAL_FONT)
-        fpick = PyFontPicker(self, wx.ID_ANY, fnt)
+        fpick = ecpickers.PyFontPicker(self, wx.ID_ANY, fnt)
         fpick.SetToolTipString(_("Main display font for various UI components"))
 
-        # Misc
-        trans_size = (-1, -1)
-        if wx.Platform == '__WXGTK__':
-            trans_size = (200, 15)
-
         # Layout
-        sizer = wx.FlexGridSizer(15, 2, 5, 5)
+        sizer = wx.FlexGridSizer(16, 2, 5, 5)
         sizer.AddMany([((10, 10), 0), ((10, 10), 0),
                        (wx.StaticText(self, label=_("Icons") + u": "), 0,
                         wx.ALIGN_CENTER_VERTICAL), (iconsz, 0),
                        ((5, 5), 0), (tbiconsz, 0),
+                       ((5, 5), 0), (tabicon_cb, 0),
                        ((10, 10), 0), ((10, 10), 0),
                        (wx.StaticText(self, label=_("Layout") + u": "), 0,
                         wx.ALIGN_CENTER_VERTICAL),
@@ -988,8 +1028,8 @@ class AppearancePanel(PrefPanelBase):
                        (wx.StaticText(self, label=_("Transparency") + u": "), 0),
                        (wx.Slider(self, ed_glob.ID_TRANSPARENCY,
                                    Profile_Get('ALPHA'), 100, 255,
-                                   size=trans_size, style=wx.SL_HORIZONTAL | \
-                                   wx.SL_AUTOTICKS | wx.SL_LABELS), 0, wx.EXPAND),
+                                   style=wx.SL_HORIZONTAL|wx.SL_AUTOTICKS|\
+                                         wx.SL_LABELS), 0, wx.EXPAND),
                        ((10, 10), 0), ((10, 10), 0),
                        (wx.StaticText(self, label=_("Display Font") + u": "),
                         0, wx.ALIGN_CENTER_VERTICAL),
@@ -1011,6 +1051,9 @@ class AppearancePanel(PrefPanelBase):
         evalue = evt.GetEventObject().GetValue()
         if e_id in (ed_glob.ID_PREF_WPOS, ed_glob.ID_PREF_WSIZE):
             Profile_Set(ed_glob.ID_2_PROF[e_id], evalue)
+        elif e_id == ed_glob.ID_PREF_TABICON:
+            Profile_Set(ed_glob.ID_2_PROF[e_id], evalue)
+            ed_msg.PostMessage(ed_msg.EDMSG_THEME_NOTEBOOK)
         elif e_id in (ed_glob.ID_SHOW_SB, ed_glob.ID_VIEW_TOOL):
             Profile_Set(ed_glob.ID_2_PROF[e_id], evalue)
             if e_id == ed_glob.ID_SHOW_SB:
@@ -1074,17 +1117,14 @@ class AppearancePanel(PrefPanelBase):
 
 #-----------------------------------------------------------------------------#
 
-class NetworkPanel(PrefPanelBase):
+class NetworkPanel(wx.Panel, PreferencesPanelBase):
     """Network related configration options"""
     def __init__(self, parent):
         """Create the panel"""
-        PrefPanelBase.__init__(self, parent)
+        wx.Panel.__init__(self, parent)
+        PreferencesPanelBase.__init__(self)
 
-        # Layout
-        self.__DoLayout()
-        self.SetAutoLayout(True)
-
-    def __DoLayout(self):
+    def _DoLayout(self):
         """Do the layout of the panel
         @note: Do not call this after __init__
 
@@ -1112,13 +1152,14 @@ class NetConfigPage(wx.Panel):
         wx.Panel.__init__(self, parent)
 
         # Layout
-        self.__DoLayout()
+        self._DoLayout()
+        self.SetAutoLayout(True)
 
         # Event Handlers
         self.Bind(wx.EVT_CHECKBOX, self.OnCheck, id=ID_USE_PROXY)
         self.Bind(wx.EVT_BUTTON, self.OnApply, id=wx.ID_APPLY)
 
-    def __DoLayout(self):
+    def _DoLayout(self):
         """Layout the controls in the panel"""
         msizer = wx.BoxSizer(wx.VERTICAL)
 
@@ -1240,13 +1281,13 @@ class UpdatePage(wx.Panel):
         wx.Panel.__init__(self, parent)
 
         # Layout
-        self.__DoLayout()
+        self._DoLayout()
 
         # Event Handlers
         self.Bind(wx.EVT_BUTTON, self.OnButton)
         self.Bind(ed_event.EVT_UPDATE_TEXT, self.OnUpdateText)
 
-    def __DoLayout(self):
+    def _DoLayout(self):
         """Do the layout of the panel
         @note: Do not call this after __init__
 
@@ -1340,7 +1381,7 @@ class UpdatePage(wx.Panel):
 #-----------------------------------------------------------------------------#
 # Advanced Page
 
-class AdvancedPanel(PrefPanelBase):
+class AdvancedPanel(wx.Panel):
     """Creates a panel for holding advanced configuration options
     @summary: Contains a wx.Notebook that contains a number of pages with
               setting controls for configuring the advanced configuration
@@ -1352,13 +1393,12 @@ class AdvancedPanel(PrefPanelBase):
         @param parent: Parent window of this panel
 
         """
-        PrefPanelBase.__init__(self, parent)
+        wx.Panel.__init__(self, parent)
 
         # Layout
-        self.__DoLayout()
-        self.SetAutoLayout(True)
+        self._layout_done = False
 
-    def __DoLayout(self):
+    def _DoLayout(self):
         """Do the layout of the panel
         @note: Do not call this after __init__
 
@@ -1371,6 +1411,13 @@ class AdvancedPanel(PrefPanelBase):
         msizer.AddMany([(sizer, 1, wx.EXPAND), ((10, 10), 0)])
         self.SetSizer(msizer)
 
+    def DoSelected(self):
+        """Handle initial selection to create controls"""
+        if not self._layout_done:
+            self._layout_done = True
+            self._DoLayout()
+            self.SetAutoLayout(True)
+
 #-----------------------------------------------------------------------------#
 # Keybinding Panel
 
@@ -1382,13 +1429,13 @@ ID_KEYS = wx.NewId()
 ID_BINDING = wx.NewId()
 KEYS = ['', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M',
         'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z',
-        '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '-', '+', 'Left',
-        'Right', 'Down', 'Up', 'Home', 'End', 'Tab']
+        '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '-', '+', '/', ',',
+        '.', '[', ']', '{', '}', ':', 'Left', 'Right', 'Down', 'Up', 'Home',
+        'End', 'Enter', 'Tab']
 KEYS.extend(["F" + str(x) for x in range(1, 13)])
 
 if wx.Platform == '__WXMSW__':
     KEYS.remove('Tab')
-KEYS.sort()
 
 MODIFIERS = ['', 'Alt', 'Shift']
 if wx.Platform == '__WXMAC__':
@@ -1424,14 +1471,14 @@ class KeyBindingPanel(wx.Panel):
                 self.menumap[key] = sorted(val, _tupSort)
 
         # Layout
-        self.__DoLayout()
+        self._DoLayout()
 
         # Event Handlers
         self.Bind(wx.EVT_BUTTON, self.OnButton)
         self.Bind(wx.EVT_CHOICE, self.OnChoice)
         self.Bind(wx.EVT_LISTBOX, self.OnListBox)
 
-    def __DoLayout(self):
+    def _DoLayout(self):
         """Layout the controls"""
         msizer = wx.BoxSizer(wx.VERTICAL) # Main Sizer
         spacer = ((5, 5), 0)
@@ -1713,6 +1760,8 @@ class KeyBindingPanel(wx.Panel):
                 csel = None
             self.binder.LoadKeyProfile(csel)
             Profile_Set('KEY_PROFILE', csel)
+            ed_msg.PostMessage(ed_msg.EDMSG_MENU_REBIND)
+            self.menub.SaveKeyProfile()
             self.EnableControls(csel is not None)
         elif e_id == ID_MENUS:
             mi_listbx = self.FindWindowById(ID_MENU_ITEMS)
@@ -1757,6 +1806,7 @@ class KeyBindingPanel(wx.Panel):
             evt.Skip()
 
 #----------------------------------------------------------------------------#
+
 class ExtListCtrl(wx.ListCtrl,
                   listmix.ListCtrlAutoWidthMixin,
                   listmix.TextEditMixin,
@@ -1780,7 +1830,6 @@ class ExtListCtrl(wx.ListCtrl,
                                    wx.LC_VRULES | wx.BORDER)
 
         listmix.ListCtrlAutoWidthMixin.__init__(self)
-        listmix.TextEditMixin.__init__(self)
         elistmix.ListRowHighlighter.__init__(self)
 
         # Setup
@@ -1789,6 +1838,8 @@ class ExtListCtrl(wx.ListCtrl,
                           _("Extensions (space separated, no dots)"))
         self._extreg = syntax.ExtensionRegister()
         self._editing = None
+        self.LoadList()
+        listmix.TextEditMixin.__init__(self)
 
     def CloseEditor(self, evt=None):
         """Update list and extension register after edit window
@@ -1852,6 +1903,7 @@ class ExtListCtrl(wx.ListCtrl,
                                u'  ' + u' '.join(self._extreg[ftype]))
 
 #----------------------------------------------------------------------------#
+
 class ExChoice(wx.Choice):
     """Class to extend wx.Choice to have the GetValue
     function. This allows the application function to remain
@@ -1865,11 +1917,14 @@ class ExChoice(wx.Choice):
 
         """
         if len(choices) and isinstance(choices[0], int):
-            for ind in range(len(choices)):
-                choices[ind] = str(choices[ind])
+            choices = [ unicode(choice) for choice in choices ]
+
         wx.Choice.__init__(self, parent, cid, choices=choices)
-        if default != None:
+
+        if default != None and isinstance(default, basestring):
             self.SetStringSelection(default)
+        elif default is not None:
+            self.SetSelection(default)
 
     def GetValue(self):
         """Gets the Selected Value
@@ -1882,101 +1937,21 @@ class ExChoice(wx.Choice):
             val.lower()
         return val
 
-class PyFontPicker(wx.Panel):
-    """A slightly enhanced wx.FontPickerCtrl that displays the choosen font in
-    the text control using the choosen font as well as the font's size using
-    nicer formatting.
-
-    """
-    def __init__(self, parent, id_=wx.ID_ANY, default=wx.NullFont):
-        """Initializes the PyFontPicker
-        @param default: The font to initialize as selected in the control
-
-        """
-        wx.Panel.__init__(self, parent, id_, style=wx.NO_BORDER)
-
-        # Attributes
-        if default == wx.NullFont:
-            self._font = wx.SystemSettings_GetFont(wx.SYS_SYSTEM_FONT)
-        else:
-            self._font = default
-        self._text = wx.TextCtrl(self, style=wx.TE_CENTER | wx.TE_READONLY)
-        self._text.SetFont(default)
-        self._text.SetValue(u"%s - %dpt" % (self._font.GetFaceName(), \
-                                            self._font.GetPointSize()))
-        self._button = wx.Button(self, label=_("Set Font") + u'...')
-
-        # Layout
-        sizer = wx.BoxSizer(wx.HORIZONTAL)
-        sizer.AddMany([(self._text, 1, wx.ALIGN_CENTER_VERTICAL), ((5, 5), 0),
-                       (self._button, 0, wx.ALIGN_CENTER_VERTICAL)])
-        self.SetSizer(sizer)
-        self.SetAutoLayout(True)
-
-        # Event Handlers
-        self.Bind(wx.EVT_BUTTON, lambda evt: self.ShowFontDlg(), self._button)
-        self.Bind(wx.EVT_FONTPICKER_CHANGED, self.OnChange)
-
-    def GetFontValue(self):
-        """Gets the currently choosen font
-        @return: wx.Font
-
-        """
-        return self._font
-
-    def GetTextCtrl(self):
-        """Gets the widgets text control
-        @return: wx.TextCtrl
-
-        """
-        return self._text
-
-    def OnChange(self, evt):
-        """Updates the text control using our custom stylings after
-        the font is changed.
-        @param evt: The event that called this handler
-
-        """
-        font = evt.GetFont()
-        if font.IsNull():
-            return
-        self._font = font
-        self._text.Clear()
-        self._text.SetFont(self._font)
-        self._text.SetValue(u"%s - %dpt" % (font.GetFaceName(), \
-                                            font.GetPointSize()))
-        evt = ed_event.NotificationEvent(ed_event.edEVT_NOTIFY,
-                                         self.GetId(), self._font, self)
-        wx.PostEvent(self.GetParent(), evt)
-
-    def SetButtonLabel(self, label):
-        """Sets the buttons label"""
-        self._button.SetLabel(label)
-        self._button.Refresh()
-
-    def SetToolTipString(self, tip):
-        """Sets the tooltip of the window
-        @param tip: string
-
-        """
-        self._text.SetToolTipString(tip)
-        self._button.SetToolTipString(tip)
-        wx.Panel.SetToolTipString(self, tip)
-
-    def ShowFontDlg(self):
-        """Opens the FontDialog and processes the result"""
-        fdata = wx.FontData()
-        fdata.SetInitialFont(self._font)
-        fdlg = wx.FontDialog(self.GetParent(), fdata)
-        fdlg.ShowModal()
-        fdata = fdlg.GetFontData()
-        fdlg.Destroy()
-        wx.PostEvent(self, wx.FontPickerEvent(self, self.GetId(),
-                                              fdata.GetChosenFont()))
-
 #-----------------------------------------------------------------------------#
 # Utility Functions
 
+def GetPrintModeStrings():
+    """Get the strings for describing the print modes
+    @note: defined in a function so translations can take place at runtime
+    @note: Order must be kept in sync with the PRINT_ vals in ed_glob
+
+    """
+    return [_('Black/White'),       # PRINT_BLACK_WHITE
+            _('Colour/White'),      # PRINT_COLOR_WHITE
+            _('Colour/Default'),    # PRINT_COLOR_DEF
+            _('Inverse'),           # PRINT_INVERSE
+            _('Normal')]            # PRINT_NORMAL
+
 def DoUpdates():
     """Update all open text controls"""
     for mainw in wx.GetApp().GetMainWindows():
