@@ -17,8 +17,8 @@ editor can load to configure the styles of the text.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: style_editor.py 55149 2008-08-21 19:18:55Z CJP $"
-__revision__ = "$Revision: 55149 $"
+__svnid__ = "$Id: style_editor.py 57242 2008-12-11 04:15:43Z CJP $"
+__revision__ = "$Revision: 57242 $"
 
 #--------------------------------------------------------------------------#
 # Imports
@@ -271,8 +271,8 @@ class StyleEditor(wx.Dialog):
 
         """
         if ed_glob.CONFIG['STYLES_DIR'] == ed_glob.CONFIG['SYS_STYLES_DIR']:
-            user_config = os.path.join(wx.GetHomeDir(),
-                                        "." + ed_glob.PROG_NAME, 'styles')
+            stdpath = wx.StandardPaths_Get()
+            user_config = os.path.join(stdpath.GetUserDataDir(), 'styles')
             if not os.path.exists(user_config):
                 try:
                     os.mkdir(user_config)
@@ -281,15 +281,20 @@ class StyleEditor(wx.Dialog):
                 else:
                     ed_glob.CONFIG['STYLES_DIR'] = user_config
 
-        dlg = wx.FileDialog(self, _("Export Style Sheet"),
-                            ed_glob.CONFIG['STYLES_DIR'],
-                            wildcard=_("Editra Style Sheet") + " (*.ess)|*.ess",
-                            style=wx.FD_SAVE | wx.OVERWRITE_PROMPT)
-        dlg.CenterOnParent()
-        result = dlg.ShowModal()
+        result = wx.ID_CANCEL
+        ss_c = self.FindWindowById(ed_glob.ID_PREF_SYNTHEME)
+        new_cb = self.FindWindowById(wx.ID_NEW)
+        if new_cb.GetValue():
+            name = ''
+        else:
+            name = ss_c.GetStringSelection()
 
-        if result == wx.ID_OK:
-            sheet_path = dlg.GetPath()
+        fname = wx.GetTextFromUser(_("Enter style sheet name"),
+                                   _("Export Style Sheet"),
+                                   name, self)
+
+        if len(fname):
+            sheet_path = os.path.join(ed_glob.CONFIG['STYLES_DIR'], fname)
             if sheet_path.split(u'.')[-1] != u'ess':
                 sheet_path += u".ess"
 
@@ -303,19 +308,19 @@ class StyleEditor(wx.Dialog):
             else:
                 # Update Style Sheet Control
                 sheet = ".".join(os.path.basename(sheet_path).split(u'.')[:-1])
-                ss_c = self.FindWindowById(ed_glob.ID_PREF_SYNTHEME)
                 ss_c.SetItems(util.GetResourceFiles(u'styles', get_all=True))
                 ss_c.SetStringSelection(sheet)
                 ss_c.Enable()
                 self.FindWindowById(wx.ID_NEW).SetValue(False)
                 self.styles_orig = self.styles_new
                 self.styles_new = DuplicateStyleDict(self.styles_orig)
+                result = wx.ID_OK
 
                 if sheet_path.startswith(ed_glob.CONFIG['STYLES_DIR']) or \
                    sheet_path.startswith(ed_glob.CONFIG['SYS_STYLES_DIR']):
                     # Update editor windows/buffer to use new style sheet
                     UpdateBufferStyles(sheet)
-        dlg.Destroy()
+
         return result
 
     def GenerateStyleSheet(self):
@@ -451,7 +456,7 @@ class StyleEditor(wx.Dialog):
         style_id = self.preview.GetStyleAt(self.preview.GetCurrentPos())
         tag_lst = self.FindWindowById(ID_STYLES)
         data = self.preview.FindTagById(style_id)
-        if data != wx.EmptyString and self.styles_new.has_key(data):
+        if data != wx.EmptyString and data in self.styles_new:
             tag_lst.SetStringSelection(data)
             if wx.Platform == '__WXGTK__':
                 tag_lst.SetFirstItemStr(data)
@@ -465,7 +470,7 @@ class StyleEditor(wx.Dialog):
 
         """
         tag = evt.GetEventObject().GetStringSelection()
-        if tag != wx.EmptyString and self.styles_new.has_key(tag):
+        if tag != wx.EmptyString and tag in self.styles_new:
             self.UpdateSettingsPane(self.styles_new[tag])
             self.EnableSettings()
         else:
