@@ -12,14 +12,16 @@ This file contains various helper functions and utilities that the program uses.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: util.py 55858 2008-09-25 12:50:12Z CJP $"
-__revision__ = "$Revision: 55858 $"
+__svnid__ = "$Id: util.py 58724 2009-02-07 20:57:44Z CJP $"
+__revision__ = "$Revision: 58724 $"
 
 #--------------------------------------------------------------------------#
-# Dependancies
+# Imports
 import os
 import sys
 import stat
+import types
+import mimetypes
 import encodings
 import codecs
 import urllib2
@@ -30,6 +32,8 @@ import ed_glob
 import ed_event
 import ed_crypt
 import dev_tool
+import syntax.syntax as syntax
+import syntax.synglob as synglob
 
 _ = wx.GetTranslation
 #--------------------------------------------------------------------------#
@@ -135,21 +139,26 @@ class DropTargetFT(wx.PyDropTarget):
                is not moved.
 
         """
+        stc = self.window
         if self._tmp is None:
-            return drag_result
+            if hasattr(stc, 'DoDragOver'):
+                val = stc.DoDragOver(x_cord, y_cord, drag_result)
+                self.ScrollBuffer(stc, x_cord, y_cord)
+            drag_result = wx.DragCopy
         else:
-            stc = self.window
-            point = wx.Point(x_cord, y_cord)
-            self._tmp.BeginDrag(point - self._lastp, stc)
-            self._tmp.Hide()
-            stc.GotoPos(stc.PositionFromPoint(point))
-            stc.Refresh()
-            stc.Update()
-            self._tmp.Move(point)
-            self._tmp.Show()
-            self._tmp.RedrawImage(self._lastp, point, True, True)
-            self._lastp = point
-            return drag_result
+            if hasattr(stc, 'DoDragOver'):
+                point = wx.Point(x_cord, y_cord)
+                self._tmp.BeginDrag(point - self._lastp, stc)
+                self._tmp.Hide()
+                stc.DoDragOver(x_cord, y_cord, drag_result)
+                self._tmp.Move(point)
+                self._tmp.Show()
+                self._tmp.RedrawImage(self._lastp, point, True, True)
+                self._lastp = point
+                self.ScrollBuffer(stc, x_cord, y_cord)
+            drag_result = wx.DragCopy
+
+        return drag_result
 
     def OnData(self, x_cord, y_cord, drag_result):
         """Gets and processes the dropped data
@@ -165,8 +174,8 @@ class DropTargetFT(wx.PyDropTarget):
         except wx.PyAssertionError:
             wx.PostEvent(self.window.GetTopLevelParent(), \
                         ed_event.StatusEvent(ed_event.edEVT_STATUS, -1,
-                                             _("Unable to open dropped file or "
-                                               "text")))
+                                             _("Unable to accept dropped file "
+                                               "or text")))
             data = False
             drag_result = wx.DragCancel
 
@@ -175,16 +184,11 @@ class DropTargetFT(wx.PyDropTarget):
             text = self._data['tdata'].GetText()
             if len(files) > 0 and self._data['fcallb'] is not None:
                 self._data['fcallb'](files)
-            elif(len(text) > 0):
-                if hasattr(self.window, 'PositionFromPointClose'):
-                    pos = self.window.PositionFromPointClose(x_cord, y_cord)
+            elif len(text) > 0:
+                if self._data['tcallb'] is not None:
+                    self._data['tcallb'](text)
                 else:
-                    pos = wx.stc.STC_INVALID_POSITION
-
-                if pos != wx.stc.STC_INVALID_POSITION:
-                    self.window.InsertText(pos, text)
-                else:
-                    drag_result = wx.DragCancel
+                    self.window.DoDropText(x_cord, y_cord, text)
         self.InitObjects()
         return drag_result
 
@@ -203,6 +207,28 @@ class DropTargetFT(wx.PyDropTarget):
             except wx.PyAssertionError, msg:
                 Log("[droptargetft][err] %s" % str(msg))
 
+    @staticmethod
+    def ScrollBuffer(stc, x_cord, y_cord):
+        """Scroll the buffer as the dragged text is moved towards the
+        ends.
+        @param stc: StyledTextCtrl
+        @param x_cord: int (x position)
+        @param y_cord: int (y position)
+        @note: currenly does not work on wxMac
+
+        """
+        cline = stc.PositionFromPoint(wx.Point(x_cord, y_cord))
+        if cline != wx.stc.STC_INVALID_POSITION:
+            cline = stc.LineFromPosition(cline)
+            fline = stc.GetFirstVisibleLine()
+            lline = stc.GetLastVisibleLine()
+            if (cline - fline) < 2:
+                stc.ScrollLines(-1)
+            elif lline - cline < 2:
+                stc.ScrollLines(1)
+            else:
+                pass
+
 #---- End FileDropTarget ----#
 
 #---- Misc Common Function Library ----#
@@ -282,10 +308,32 @@ def GetFileModTime(file_name):
     """
     try:
         mod_time = os.path.getmtime(file_name)
-    except EnvironmentError:
+    except (OSError, EnvironmentError):
         mod_time = 0
     return mod_time
 
+def GetFileType(fname):
+    """Get what the type of the file is as Editra sees it
+    in a formatted string.
+    @param fname: file path
+    @return: string (formatted/translated filetype)
+
+    """
+    if os.path.isdir(fname):
+        return _("Folder")
+
+    eguess = syntax.GetTypeFromExt(fname.split('.')[-1])
+    if eguess == synglob.LANG_TXT and fname.split('.')[-1] == 'txt':
+        return _("Text Document")
+    elif eguess == synglob.LANG_TXT:
+        mtype = mimetypes.guess_type(fname)[0]
+        if mtype is not None:
+            return mtype
+        else:
+            return _("Unknown")
+    else:
+        return _("%s Source File") % eguess
+
 def GetFileReader(file_name, enc='utf-8'):
     """Returns a file stream reader object for reading the
     supplied file name. It returns a file reader using the encoding
@@ -323,7 +371,7 @@ def GetFileWriter(file_name, enc='utf-8'):
 
     """
     try:
-        file_h = file(file_name, "wb")
+        file_h = open(file_name, "wb")
     except IOError:
         dev_tool.DEBUGP("[file_writer][err] Failed to open file %s" % file_name)
         return -1
@@ -368,8 +416,8 @@ def GetPathName(path):
 
 def GetFileManagerCmd():
     """Get the file manager open command for the current os. Under linux
-    it will check for nautilus and konqueror and return which one it finds
-    first or 'nautilus' (Gnome) if it finds neither.
+    it will check for xdg-open, nautilus, konqueror, and Thunar, it will then
+    return which one it finds first or 'nautilus' it finds nothing.
     @return: string
 
     """
@@ -452,31 +500,6 @@ def MakeNewFolder(path, name):
 
     return (True, folder)
 
-def ResolvAbsPath(rel_path):
-    """Takes a relative path and converts it to an
-    absolute path.
-    @param rel_path: path to construct absolute path for
-
-    """
-    cwd = os.getcwd()
-    pieces = rel_path.split(os.sep)
-    cut = 0
-
-    for token in pieces:
-        if token == "..":
-            cut += 1
-
-        if cut > 0:
-            rpath = os.sep.join(pieces[cut:])
-            cut *= -1
-            cwd = cwd.split(os.sep)
-            apath = os.sep.join(cwd[0:cut])
-        else:
-            rpath = rel_path
-            apath = cwd
-
-    return apath + os.sep + rpath
-
 def HasConfigDir(loc=u""):
     """ Checks if the user has a config directory and returns True
     if the config directory exists or False if it does not.
@@ -485,15 +508,10 @@ def HasConfigDir(loc=u""):
     """
     cbase = ed_glob.CONFIG['CONFIG_BASE']
     if cbase is None:
-        to_check = u"%s%s.%s%s%s" % (wx.GetHomeDir(), os.sep,
-                                     ed_glob.PROG_NAME, os.sep, loc)
-    else:
-        to_check = cbase + os.sep + loc
+        cbase = wx.StandardPaths_Get().GetUserDataDir() + os.sep
 
-    if os.path.exists(to_check):
-        return True
-    else:
-        return False
+    to_check = os.path.join(cbase, loc)
+    return os.path.exists(to_check)
 
 def MakeConfigDir(name):
     """Makes a user config directory
@@ -502,12 +520,10 @@ def MakeConfigDir(name):
     """
     cbase = ed_glob.CONFIG['CONFIG_BASE']
     if cbase is None:
-        config_dir = wx.GetHomeDir() + os.sep + u"." + ed_glob.PROG_NAME
-    else:
-        config_dir = cbase
+        cbase = wx.StandardPaths_Get().GetUserDataDir()
 
     try:
-        os.mkdir(config_dir + os.sep + name)
+        os.mkdir(cbase + os.sep + name)
     except (OSError, IOError):
         pass
 
@@ -518,14 +534,13 @@ def CreateConfigDir():
 
     """
     #---- Resolve Paths ----#
-    cbase = ed_glob.CONFIG['CONFIG_BASE']
-    if cbase is None:
-        config_dir = u"%s%s.%s" % (wx.GetHomeDir(), os.sep, ed_glob.PROG_NAME)
-    else:
-        config_dir = cbase
-    profile_dir = u"%s%sprofiles" % (config_dir, os.sep)
-    dest_file = u"%s%sdefault.ppb" % (profile_dir, os.sep)
-    ext_cfg = ["cache", "styles", "plugins"]
+    config_dir = ed_glob.CONFIG['CONFIG_BASE']
+    if config_dir is None:
+        config_dir = wx.StandardPaths_Get().GetUserDataDir() + os.sep
+
+    profile_dir = os.path.join(config_dir, u"profiles")
+    dest_file = os.path.join(profile_dir, u"default.ppb")
+    ext_cfg = [u"cache", u"styles", u"plugins"]
 
     #---- Create Directories ----#
     if not os.path.exists(config_dir):
@@ -556,17 +571,15 @@ def ResolvConfigDir(config_dir, sys_only=False):
            the code has proven itself.
 
     """
+    stdpath = wx.StandardPaths_Get()
+
+    # Try to get a User config directory
     if not sys_only:
-        # If the Config Base has already been determined use it instead of doing
-        # a full search.
-        cbase = ed_glob.CONFIG['CONFIG_BASE']
-        if cbase is None:
-            # Try to look for a user dir
-            user_config = u"%s%s.%s%s%s" % (wx.GetHomeDir(), os.sep,
-                                            ed_glob.PROG_NAME, os.sep,
-                                            config_dir)
-        else:
-            user_config = os.path.normpath(cbase + os.sep + config_dir)
+        user_config = ed_glob.CONFIG['CONFIG_BASE']
+        if user_config is None:
+            user_config = stdpath.GetUserDataDir() + os.sep
+
+        user_config = os.path.join(user_config, config_dir)
 
         if os.path.exists(user_config):
             return user_config + os.sep
@@ -574,15 +587,20 @@ def ResolvConfigDir(config_dir, sys_only=False):
     # The following lines are used only when Editra is being run as a
     # source package. If the found path does not exist then Editra is
     # running as as a built package.
-    path = __file__
-    path = os.sep.join(path.split(os.sep)[:-2])
-    path =  path + os.sep + config_dir + os.sep
-    if os.path.exists(path):
-        return path
+    if not hasattr(sys, 'frozen'):
+        path = __file__
+        path = os.sep.join(path.split(os.sep)[:-2])
+        path =  path + os.sep + config_dir + os.sep
+        if os.path.exists(path):
+            if not isinstance(path, types.UnicodeType):
+                path = unicode(path, sys.getfilesystemencoding())
+            return path
 
     # If we get here we need to do some platform dependant lookup
     # to find everything.
     path = sys.argv[0]
+    if not isinstance(path, types.UnicodeType):
+        path = unicode(path, sys.getfilesystemencoding())
 
     # If it is a link get the real path
     if os.path.islink(path):
@@ -602,28 +620,34 @@ def ResolvConfigDir(config_dir, sys_only=False):
             pieces = pro_path.split(os.sep)
             pro_path = os.sep.join(pieces[:-1])
         else:
-            pro_path = ResolvAbsPath(pro_path)
+            pro_path = os.path.abspath(pro_path)
+    elif os.sys.platform == "darwin":
+        # On OS X the config directories are in the applet under Resources
+        pro_path = stdpath.GetResourcesDir()
+        pro_path = os.path.join(pro_path, config_dir)
     else:
         pro_path = os.sep.join(pieces[:-2])
 
         if pro_path.startswith(os.sep):
             pass
-        elif pro_path == "":
+        elif pro_path == u"":
             pro_path = os.getcwd()
             pieces = pro_path.split(os.sep)
             if pieces[-1] not in [ed_glob.PROG_NAME.lower(), ed_glob.PROG_NAME]:
                 pro_path = os.sep.join(pieces[:-1])
         else:
-            pro_path = ResolvAbsPath(pro_path)
+            pro_path = os.path.abspath(pro_path)
 
-    if os.sys.platform == "darwin":
-        # On OS X the config directories are in the applet under Resources
-        pro_path = u"%s%sResources%s%s%s" % (pro_path, os.sep, os.sep,
-                                             config_dir, os.sep)
-    else:
+    if os.sys.platform != "darwin":
         pro_path = pro_path + os.sep + config_dir + os.sep
 
-    return os.path.normpath(pro_path) + os.sep
+    path = os.path.normpath(pro_path) + os.sep
+
+    # Make sure path is unicode
+    if not isinstance(path, types.UnicodeType):
+        path = unicode(path, sys.getdefaultencoding())
+
+    return path
 
 def GetResources(resource):
     """Returns a list of resource directories from a given toplevel config dir
